@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.42 2009/01/21 22:47:31 nicm Exp $ */
+/* $Id: client.c,v 1.46 2009/03/31 22:20:42 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -21,6 +21,7 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
+#include <sys/wait.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -35,8 +36,7 @@
 void	client_handle_winch(struct client_ctx *);
 
 int
-client_init(
-    const char *path, struct client_ctx *cctx, int start_server, int flags)
+client_init(char *path, struct client_ctx *cctx, int start_server, int flags)
 {
 	struct sockaddr_un		sa;
 	struct stat			sb;
@@ -47,7 +47,7 @@ client_init(
 	struct buffer		       *b;
 	char			       *name;
 
-	if (stat(path, &sb) != 0) {
+	if (lstat(path, &sb) != 0) {
 		if (start_server && errno == ENOENT) {
 			if ((cctx->srv_fd = server_start(path)) == -1)
 				goto start_failed;
@@ -85,9 +85,9 @@ client_init(
 
 server_started:
 	if ((mode = fcntl(cctx->srv_fd, F_GETFL)) == -1)
-		fatal("fcntl");
+		fatal("fcntl failed");
 	if (fcntl(cctx->srv_fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl");
+		fatal("fcntl failed");
 	cctx->srv_in = buffer_create(BUFSIZ);
 	cctx->srv_out = buffer_create(BUFSIZ);
 
@@ -142,6 +142,10 @@ client_main(struct client_ctx *cctx)
 	error = NULL;
 	xtimeout = INFTIM;
 	while (!sigterm) {
+		if (sigchld) {
+			waitpid(WAIT_ANY, NULL, WNOHANG);
+			sigchld = 0;
+		}
 		if (sigwinch)
 			client_handle_winch(cctx);
 		if (sigcont) {
