@@ -1,4 +1,4 @@
-/* $Id: cmd-attach-session.c,v 1.21 2008/12/10 20:25:41 nicm Exp $ */
+/* $Id: cmd-attach-session.c,v 1.24 2009/01/23 16:59:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,12 +24,12 @@
  * Attach existing session to the current terminal.
  */
 
-void	cmd_attach_session_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_attach_session_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_attach_session_entry = {
 	"attach-session", "attach",
 	"[-d] " CMD_TARGET_SESSION_USAGE,
-	CMD_DFLAG|CMD_CANTNEST,
+       	CMD_DFLAG|CMD_CANTNEST|CMD_STARTSERVER,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_attach_session_exec,
@@ -39,28 +39,32 @@ const struct cmd_entry cmd_attach_session_entry = {
 	cmd_target_print
 };
 
-void
+int
 cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
 	struct session		*s;
 	char			*cause;
-
+	
 	if (ctx->curclient != NULL)
-		return;
+		return (0);
 
+	if (ARRAY_LENGTH(&sessions) == 0) {
+		ctx->error(ctx, "no sessions");
+		return (-1);
+	}
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
-		return;
+		return (-1);
 
 	if (!(ctx->cmdclient->flags & CLIENT_TERMINAL)) {
 		ctx->error(ctx, "not a terminal");
-		return;
+		return (-1);
 	}
 
 	if (tty_open(&ctx->cmdclient->tty, &cause) != 0) {
-		ctx->error(ctx, "%s", cause);
+		ctx->error(ctx, "terminal open failed: %s", cause);
 		xfree(cause);
-		return;
+		return (-1);
 	}
 
 	if (data->flags & CMD_DFLAG)
@@ -70,5 +74,7 @@ cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	server_write_client(ctx->cmdclient, MSG_READY, NULL, 0);
 	recalculate_sizes();
 	server_redraw_client(ctx->cmdclient);
+
+	return (1);
 }
 
