@@ -1,4 +1,4 @@
-/* $Id: cmd-command-prompt.c,v 1.13 2009/01/30 21:10:10 nicm Exp $ */
+/* $Id: cmd-command-prompt.c,v 1.16 2009/02/16 18:58:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -96,7 +96,7 @@ cmd_command_prompt_exec(struct cmd *self, struct cmd_ctx *ctx)
 		cdata->template = NULL;
 		hdr = xstrdup(":");
 	}
-	server_set_client_prompt(c, hdr, cmd_command_prompt_callback, cdata, 0);
+	status_prompt_set(c, hdr, cmd_command_prompt_callback, cdata, 0);
 	xfree(hdr);
 
 	return (0);
@@ -112,8 +112,10 @@ cmd_command_prompt_callback(void *data, const char *s)
 	char				*cause, *ptr, *buf, ch;
 	size_t				 len, slen;
 
-	if (s == NULL)
+	if (s == NULL) {
+		xfree(cdata);
 		return (0);
+	}
 	slen = strlen(s);
 
 	len = 0;
@@ -148,7 +150,7 @@ cmd_command_prompt_callback(void *data, const char *s)
 		if (cause == NULL)
 			return (0);
 		*cause = toupper((u_char) *cause);
-		server_set_client_message(c, cause);
+		status_message_set(c, cause);
 		xfree(cause);
 		cmdlist = NULL;
 	}
@@ -168,6 +170,7 @@ cmd_command_prompt_callback(void *data, const char *s)
 	ctx.cmdclient = NULL;
 
 	cmd_list_exec(cmdlist, &ctx);
+	cmd_list_free(cmdlist);
 
 	if (c->prompt_callback != (void *) &cmd_command_prompt_callback)
 		return (1);
