@@ -1,4 +1,4 @@
-/* $Id: cmd-link-window.c,v 1.26 2008/12/10 20:25:41 nicm Exp $ */
+/* $Id: cmd-link-window.c,v 1.28 2009/01/23 16:59:14 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,7 +26,7 @@
  * Link a window into another session.
  */
 
-void	cmd_link_window_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_link_window_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_link_window_entry = {
 	"link-window", "linkw",
@@ -41,20 +41,21 @@ const struct cmd_entry cmd_link_window_entry = {
 	cmd_srcdst_print
 };
 
-void
+int
 cmd_link_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_srcdst_data	*data = self->data;
 	struct session		*dst;
 	struct winlink		*wl_src, *wl_dst;
+	char			*cause;
 	int			 idx;
 
 	if ((wl_src = cmd_find_window(ctx, data->src, NULL)) == NULL)
-		return;
+		return (-1);
 
 	if (arg_parse_window(data->dst, &dst, &idx) != 0) {
 		ctx->error(ctx, "bad window: %s", data->dst);
-		return;
+		return (-1);
 	}
 	if (dst == NULL)
 		dst = ctx->cursession;
@@ -62,7 +63,7 @@ cmd_link_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		dst = cmd_current_session(ctx);
 	if (dst == NULL) {
 		ctx->error(ctx, "session not found: %s", data->dst);
-		return;
+		return (-1);
 	}
 
 	wl_dst = NULL;
@@ -70,7 +71,7 @@ cmd_link_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		wl_dst = winlink_find_by_index(&dst->windows, idx);
 	if (wl_dst != NULL) {
 		if (wl_dst->window == wl_src->window)
-			goto out;
+			return (0);
 
 		if (data->flags & CMD_KFLAG) {
 			/*
@@ -89,10 +90,11 @@ cmd_link_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 	}
 
-	wl_dst = session_attach(dst, wl_src->window, idx);
+	wl_dst = session_attach(dst, wl_src->window, idx, &cause);
 	if (wl_dst == NULL) {
-		ctx->error(ctx, "index in use: %d", idx);
-		return;
+		ctx->error(ctx, "create session failed: %s", cause);
+		xfree(cause);
+		return (-1);
 	}
 
 	if (data->flags & CMD_DFLAG)
@@ -103,7 +105,5 @@ cmd_link_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	}
 	recalculate_sizes();
 
-out:
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
