@@ -1,4 +1,4 @@
-/* $Id: cmd-list-windows.c,v 1.30 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-list-windows.c,v 1.33 2009/03/28 20:17:29 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -49,7 +49,7 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct winlink		*wl;
 	struct window		*w;
 	struct window_pane	*wp;	
-	struct grid_data	*gd;
+	struct grid		*gd;
 	u_int			 i;
 	unsigned long long	 size;
 	const char		*name;
@@ -66,8 +66,10 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 			gd = wp->base.grid;
 			
 			size = 0;
-			for (i = 0; i < gd->hsize; i++)
+			for (i = 0; i < gd->hsize; i++) {
 				size += gd->size[i] * sizeof **gd->data;
+				size += gd->usize[i] * sizeof **gd->udata;
+			}
 			size += gd->hsize * (sizeof *gd->data);
 			size += gd->hsize * (sizeof *gd->size);
 
