@@ -1,4 +1,4 @@
-/* $Id: cmd-resize-pane-down.c,v 1.6 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-resize-pane-down.c,v 1.8 2009/04/02 21:11:52 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -32,7 +32,7 @@ int	cmd_resize_pane_down_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_resize_pane_down_entry = {
 	"resize-pane-down", "resizep-down",
 	CMD_PANE_WINDOW_USAGE " [adjustment]",
-	CMD_ARG01|CMD_CANREPEAT,
+	CMD_ARG01,
 	cmd_resize_pane_down_init,
 	cmd_pane_parse,
 	cmd_resize_pane_down_exec,
@@ -65,6 +65,10 @@ cmd_resize_pane_down_exec(struct cmd *self, struct cmd_ctx *ctx)
 	
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
 		return (-1);
+	if (wl->window->layout != 0) {
+		ctx->error(ctx, "window not in manual layout");
+		return (-1);
+	}
 	if (data->pane == -1)
 		wp = wl->window->active;
 	else {
