@@ -1,4 +1,4 @@
-/* $Id: cmd-send-keys.c,v 1.16 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: cmd-send-keys.c,v 1.18 2009/01/19 18:23:40 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,11 +27,11 @@
  */
 
 int	cmd_send_keys_parse(struct cmd *, int, char **, char **);
-void	cmd_send_keys_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_send_keys_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_send_keys_send(struct cmd *, struct buffer *);
 void	cmd_send_keys_recv(struct cmd *, struct buffer *);
 void	cmd_send_keys_free(struct cmd *);
-void	cmd_send_keys_print(struct cmd *, char *, size_t);
+size_t	cmd_send_keys_print(struct cmd *, char *, size_t);
 
 struct cmd_send_keys_data {
 	char	*target;
@@ -106,7 +106,7 @@ usage:
 	return (-1);
 }
 
-void
+int
 cmd_send_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_send_keys_data	*data = self->data;
@@ -114,18 +114,17 @@ cmd_send_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 	u_int				 i;
 
 	if (data == NULL)
-		return;
+		return (-1);
 
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
-		return;
+		return (-1);
 
 	for (i = 0; i < data->nkeys; i++) {
 		window_pane_key(
 		    wl->window->active, ctx->curclient, data->keys[i]);
 	}
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
 
 void
@@ -160,7 +159,7 @@ cmd_send_keys_free(struct cmd *self)
 	xfree(data);
 }
 
-void
+size_t
 cmd_send_keys_print(struct cmd *self, char *buf, size_t len)
 {
 	struct cmd_send_keys_data	*data = self->data;
@@ -169,9 +168,9 @@ cmd_send_keys_print(struct cmd *self, char *buf, size_t len)
 
 	off += xsnprintf(buf, len, "%s", self->entry->name);
 	if (data == NULL)
-		return;
+		return (off);
 	if (off < len && data->target != NULL)
-		off += xsnprintf(buf + off, len - off, " -t %s", data->target);
+		off += cmd_prarg(buf + off, len - off, " -t ", data->target);
 	if (off < len && data->idx != -1)
 		off += xsnprintf(buf + off, len - off, " -i %d", data->idx);
 
@@ -181,4 +180,5 @@ cmd_send_keys_print(struct cmd *self, char *buf, size_t len)
 		off += xsnprintf(buf + off,
 		    len - off, " %s", key_string_lookup_key(data->keys[i]));
 	}
+	return (off);
 }
