@@ -1,4 +1,4 @@
-/* $Id: cmd-set-option.c,v 1.58 2009/01/30 00:24:49 nicm Exp $ */
+/* $Id: cmd-set-option.c,v 1.60 2009/03/21 12:44:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -42,6 +42,9 @@ const struct cmd_entry cmd_set_option_entry = {
 	cmd_option_print
 };
 
+const char *set_option_status_keys_list[] = {
+	"emacs", "vi", NULL
+};
 const char *set_option_bell_action_list[] = {
 	"none", "any", "current", NULL
 };
@@ -58,12 +61,14 @@ const struct set_option_entry set_option_table[NSETOPTION] = {
 	{ "message-fg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "prefix", SET_OPTION_KEY, 0, 0, NULL },
 	{ "repeat-time", SET_OPTION_NUMBER, 0, SHRT_MAX, NULL },
+	{ "set-remain-on-exit", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "set-titles", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "status", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "status-attr", SET_OPTION_ATTRIBUTES, 0, 0, NULL },
 	{ "status-bg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "status-fg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "status-interval", SET_OPTION_NUMBER, 0, INT_MAX, NULL },
+	{ "status-keys", SET_OPTION_CHOICE, 0, 0, set_option_status_keys_list },
 	{ "status-left", SET_OPTION_STRING, 0, 0, NULL },
 	{ "status-left-length", SET_OPTION_NUMBER, 0, SHRT_MAX, NULL },
 	{ "status-right", SET_OPTION_STRING, 0, 0, NULL },
