@@ -1,4 +1,4 @@
-/* $Id: cmd-set-window-option.c,v 1.19 2009/01/10 19:35:39 nicm Exp $ */
+/* $Id: cmd-set-window-option.c,v 1.24 2009/01/30 00:24:49 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,11 +28,11 @@
  */
 
 int	cmd_set_window_option_parse(struct cmd *, int, char **, char **);
-void	cmd_set_window_option_exec(struct cmd *, struct cmd_ctx *);
+int	cmd_set_window_option_exec(struct cmd *, struct cmd_ctx *);
 void	cmd_set_window_option_send(struct cmd *, struct buffer *);
 void	cmd_set_window_option_recv(struct cmd *, struct buffer *);
 void	cmd_set_window_option_free(struct cmd *);
-void	cmd_set_window_option_print(struct cmd *, char *, size_t);
+size_t	cmd_set_window_option_print(struct cmd *, char *, size_t);
 
 const struct cmd_entry cmd_set_window_option_entry = {
 	"set-window-option", "setw",
@@ -55,21 +55,26 @@ const char *set_option_clock_mode_style_list[] = {
 };
 const struct set_option_entry set_window_option_table[NSETWINDOWOPTION] = {
 	{ "aggressive-resize", SET_OPTION_FLAG, 0, 0, NULL },
+	{ "automatic-rename", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "clock-mode-colour", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "clock-mode-style",
 	  SET_OPTION_CHOICE, 0, 0, set_option_clock_mode_style_list },
 	{ "force-height", SET_OPTION_NUMBER, 0, INT_MAX, NULL },
 	{ "force-width", SET_OPTION_NUMBER, 0, INT_MAX, NULL },
+	{ "mode-attr", SET_OPTION_ATTRIBUTES, 0, 0, NULL },
 	{ "mode-bg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "mode-fg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "mode-keys", SET_OPTION_CHOICE, 0, 0, set_option_mode_keys_list },
 	{ "monitor-activity", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "remain-on-exit", SET_OPTION_FLAG, 0, 0, NULL },
 	{ "utf8", SET_OPTION_FLAG, 0, 0, NULL },
+	{ "window-status-attr", SET_OPTION_ATTRIBUTES, 0, 0, NULL },
+	{ "window-status-bg", SET_OPTION_COLOUR, 0, 0, NULL },
+	{ "window-status-fg", SET_OPTION_COLOUR, 0, 0, NULL },
 	{ "xterm-keys", SET_OPTION_FLAG, 0, 0, NULL },
 };
 
-void
+int
 cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_option_data		*data = self->data;
@@ -83,13 +88,13 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 		oo = &global_window_options;
 	else {
 		if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
-			return;
+			return (-1);
 		oo = &wl->window->options;
 	}
 
 	if (*data->option == '\0') {
 		ctx->error(ctx, "invalid option");
-		return;
+		return (-1);
 	}
 
 	entry = NULL;
@@ -99,7 +104,7 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 			continue;
 		if (entry != NULL) {
 			ctx->error(ctx, "ambiguous option: %s", data->option);
-			return;
+			return (-1);
 		}
 		entry = &set_window_option_table[i];
 
@@ -109,26 +114,22 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 	}
 	if (entry == NULL) {
 		ctx->error(ctx, "unknown option: %s", data->option);
-		return;
+		return (-1);
 	}
 
 	if (data->flags & CMD_UFLAG) {
 		if (data->flags & CMD_GFLAG) {
 			ctx->error(ctx,
 			    "can't unset global option: %s", entry->name);
-			return;
+			return (-1);
 		}
 		if (data->value != NULL) {
 			ctx->error(ctx,
 			    "value passed to unset option: %s", entry->name);
-			return;
+			return (-1);
 		}
 
-		if (options_remove(oo, entry->name) != 0) {
-			ctx->error(ctx,
-			    "can't unset option, not set: %s", entry->name);
-			return;
-		}
+		options_remove(oo, entry->name);
 		ctx->info(ctx, "unset option: %s", entry->name);
 	} else {
 		switch (entry->type) {
@@ -144,6 +145,9 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 		case SET_OPTION_COLOUR:
 			set_option_colour(ctx, oo, entry, data->value);
 			break;
+		case SET_OPTION_ATTRIBUTES:
+			set_option_attributes(ctx, oo, entry, data->value);
+			break;
 		case SET_OPTION_FLAG:
 			set_option_flag(ctx, oo, entry, data->value);
 			break;
@@ -160,6 +164,5 @@ cmd_set_window_option_exec(struct cmd *self, struct cmd_ctx *ctx)
 			server_redraw_client(c);
 	}
 
-	if (ctx->cmdclient != NULL)
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+	return (0);
 }
