@@ -1,4 +1,4 @@
-/* $Id: cmd-split-window.c,v 1.8 2009/01/23 16:59:14 nicm Exp $ */
+/* $Id: cmd-split-window.c,v 1.12 2009/04/03 17:21:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,7 +24,7 @@
 #include "tmux.h"
 
 /*
- * Create a new window.
+ * Split a window (add a new pane).
  */
 
 int	cmd_split_window_parse(struct cmd *, int, char **, char **);
@@ -139,18 +139,15 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct winlink			*wl;
 	struct window			*w;
 	struct window_pane		*wp;
-	const char			*env[] = CHILD_ENVIRON;
-	char		 		 buf[256], *cmd, *cwd, *cause;
-	u_int				 i, hlimit, lines;
+	const char		       **env;
+	char		 		*cmd, *cwd, *cause;
+	u_int				 hlimit, lines;
 
 	if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
 		return (-1);
 	w = wl->window;
 
-	if (session_index(s, &i) != 0)
-		fatalx("session not found");
-	xsnprintf(buf, sizeof buf, "TMUX=%ld,%u", (long) getpid(), i);
-	env[0] = buf;
+	env = server_fill_environ(s);
 
 	cmd = data->cmd;
 	if (cmd == NULL)
@@ -181,6 +178,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		server_redraw_session(s);
 	} else
 		server_status_session(s);
+	layout_refresh(w, 0);
 
 	return (0);
 }
