@@ -1,4 +1,4 @@
-/* $Id: cmd-string.c,v 1.11 2009/01/18 14:40:48 nicm Exp $ */
+/* $Id: cmd-string.c,v 1.13 2009/02/16 19:29:17 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -56,7 +56,7 @@ int
 cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 {
 	size_t		p;
-	int		ch, argc, rval;
+	int		ch, argc, rval, have_arg;
 	char	      **argv, *buf, *t, *u;
 	size_t		len;
 
@@ -77,6 +77,8 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 	buf = NULL;
 	len = 0;
 
+	have_arg = 0;
+
 	*cause = NULL;
 
 	*cmdlist = NULL;
@@ -92,6 +94,9 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			buf = xrealloc(buf, 1, len + strlen(t) + 1);
 			strlcpy(buf + len, t, strlen(t) + 1);
 			len += strlen(t);
+			xfree(t);
+
+			have_arg = 1;
 			break;
 		case '"':
 			if ((t = cmd_string_string(s, &p, '"', 1)) == NULL)
@@ -99,6 +104,9 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			buf = xrealloc(buf, 1, len + strlen(t) + 1);
 			strlcpy(buf + len, t, strlen(t) + 1);
 			len += strlen(t);
+			xfree(t);
+
+			have_arg = 1;
 			break;
 		case '$':
 			if ((t = cmd_string_variable(s, &p)) == NULL)
@@ -106,6 +114,8 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			buf = xrealloc(buf, 1, len + strlen(t) + 1);
 			strlcpy(buf + len, t, strlen(t) + 1);
 			len += strlen(t);
+
+			have_arg = 1;
 			break;
 		case '#':
 			/* Comment: discard rest of line. */
@@ -115,7 +125,7 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 		case EOF:
 		case ' ':
 		case '\t':
- 			if (len != 0) {
+ 			if (have_arg) {
 				buf = xrealloc(buf, 1, len + 1);
 				buf[len] = '\0';
 
@@ -124,6 +134,8 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 
 				buf = NULL;
 				len = 0;
+
+				have_arg = 0;
 			}
 
 			if (ch != EOF)
@@ -135,6 +147,10 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 			if (*cmdlist == NULL)
 				goto out;
 
+			do
+				xfree(argv[argc - 1]);
+			while (--argc > 0);
+
 			rval = 0;
 			goto out;
 		default:
@@ -143,6 +159,8 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, char **cause)
 
 			buf = xrealloc(buf, 1, len + 1);
 			buf[len++] = ch;
+
+			have_arg = 1;
 			break;
 		}
 	}
