@@ -1,4 +1,4 @@
-/* $Id: cmd.c,v 1.85 2009/02/03 17:21:19 tcunha Exp $ */
+/* $Id: cmd.c,v 1.89 2009/04/03 17:21:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,6 +28,7 @@
 const struct cmd_entry *cmd_table[] = {
 	&cmd_attach_session_entry,
 	&cmd_bind_key_entry,
+	&cmd_break_pane_entry,
 	&cmd_choose_session_entry,
 	&cmd_choose_window_entry,
 	&cmd_clock_mode_entry,
@@ -56,6 +57,7 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_move_window_entry,
 	&cmd_new_session_entry,
 	&cmd_new_window_entry,
+	&cmd_next_layout_entry,
 	&cmd_next_window_entry,
 	&cmd_paste_buffer_entry,
 	&cmd_previous_window_entry,
@@ -64,7 +66,8 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_rename_window_entry,
 	&cmd_resize_pane_down_entry,
 	&cmd_resize_pane_up_entry,
-	&cmd_respawn_window_entry,
+	&cmd_respawn_window_entry,	
+	&cmd_rotate_window_entry,
 	&cmd_save_buffer_entry,
 	&cmd_scroll_mode_entry,
 	&cmd_select_pane_entry,
@@ -84,6 +87,7 @@ const struct cmd_entry *cmd_table[] = {
 	&cmd_split_window_entry,
 	&cmd_start_server_entry,
 	&cmd_suspend_client_entry,
+	&cmd_swap_pane_entry,
 	&cmd_swap_window_entry,
 	&cmd_switch_client_entry,
 	&cmd_unbind_key_entry,
