@@ -1,4 +1,4 @@
-/* $Id: grid.c,v 1.7 2009/01/17 18:47:36 nicm Exp $ */
+/* $Id: grid.c,v 1.15 2009/03/30 19:44:55 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -35,18 +35,21 @@
  */
 
 /* Default grid cell data. */
-const struct grid_cell grid_default_cell = { ' ', 0, 0, 8, 8 };
+const struct grid_cell grid_default_cell = { 0, 0, 8, 8, ' ' };
 
 #define grid_put_cell(gd, px, py, gc) do {			\
 	memcpy(&gd->data[py][px], gc, sizeof gd->data[py][px]);	\
 } while (0)
+#define grid_put_utf8(gd, px, py, gc) do {			\
+	memcpy(&gd->udata[py][px], gc, sizeof gd->udata[py][px]); \
+} while (0)
 
-int	grid_check_x(struct grid_data *, u_int);
-int	grid_check_y(struct grid_data *, u_int);
+int	grid_check_x(struct grid *, u_int);
+int	grid_check_y(struct grid *, u_int);
 
 #ifdef DEBUG
 int
-grid_check_x(struct grid_data *gd, u_int px)
+grid_check_x(struct grid *gd, u_int px)
 {
 	if ((px) >= (gd)->sx)
 		log_fatalx("x out of range: %u", px);
@@ -54,7 +57,7 @@ grid_check_x(struct grid_data *gd, u_int px)
 }
 
 int
-grid_check_y(struct grid_data *gd, u_int py)
+grid_check_y(struct grid *gd, u_int py)
 {
 	if ((py) >= (gd)->hsize + (gd)->sy)
 		log_fatalx("y out of range: %u", py);
@@ -62,7 +65,7 @@ grid_check_y(struct grid_data *gd, u_int py)
 }
 #else
 int
-grid_check_x(struct grid_data *gd, u_int px)
+grid_check_x(struct grid *gd, u_int px)
 {
 	if ((px) >= (gd)->sx) {
 		log_debug("x out of range: %u", px);
@@ -72,7 +75,7 @@ grid_check_x(struct grid_data *gd, u_int px)
 }
 
 int
-grid_check_y(struct grid_data *gd, u_int py)
+grid_check_y(struct grid *gd, u_int py)
 {
 	if ((py) >= (gd)->hsize + (gd)->sy) {
 		log_debug("y out of range: %u", py);
@@ -83,10 +86,10 @@ grid_check_y(struct grid_data *gd, u_int py)
 #endif
 
 /* Create a new grid. */
-struct grid_data *
+struct grid *
 grid_create(u_int sx, u_int sy, u_int hlimit)
 {
-	struct grid_data	*gd;
+	struct grid	*gd;
 
 	gd = xmalloc(sizeof *gd);
 	gd->sx = sx;
@@ -98,30 +101,72 @@ grid_create(u_int sx, u_int sy, u_int hlimit)
 	gd->size = xcalloc(gd->sy, sizeof *gd->size);
 	gd->data = xcalloc(gd->sy, sizeof *gd->data);
 
+	gd->usize = xcalloc(gd->sy, sizeof *gd->usize);
+	gd->udata = xcalloc(gd->sy, sizeof *gd->udata);
+
 	return (gd);
 }
 
 /* Destroy grid. */
 void
-grid_destroy(struct grid_data *gd)
+grid_destroy(struct grid *gd)
 {
 	u_int	yy;
 
-	for (yy = 0; yy < gd->hsize + gd->sy - 1; yy++) {
+	for (yy = 0; yy < gd->hsize + gd->sy; yy++) {
+		if (gd->udata[yy] != NULL)
+			xfree(gd->udata[yy]);
 		if (gd->data[yy] != NULL)
 			xfree(gd->data[yy]);
 	}
 
+	if (gd->udata != NULL)
+		xfree(gd->udata);
+	if (gd->usize != NULL)
+		xfree(gd->usize);
+
 	if (gd->data != NULL)
 		xfree(gd->data);
 	if (gd->size != NULL)
 		xfree(gd->size);
+
 	xfree(gd);
 }
 
+/* Compare grids. */
+int
+grid_compare(struct grid *ga, struct grid *gb)
+{
+	struct grid_cell	*gca, *gcb;
+	struct grid_utf8	*gua, *gub;
+	u_int			 xx, yy;
+
+	if (ga->sx != gb->sx || ga->sy != ga->sy)
+		return (1);
+
+	for (yy = 0; yy < ga->sy; yy++) {
+		if (ga->size[yy] != gb->size[yy])
+			return (1);
+		for (xx = 0; xx < ga->sx; xx++) {
+			gca = &ga->data[yy][xx];
+			gcb = &gb->data[yy][xx];
+			if (memcmp(gca, gcb, sizeof (struct grid_cell)) != 0)
+				return (1);
+			if (!(gca->flags & GRID_FLAG_UTF8))
+				continue;
+			gua = &ga->udata[yy][xx];
+			gub = &gb->udata[yy][xx];
+			if (memcmp(gua, gub, sizeof (struct grid_utf8)) != 0)
+				return (1);
+		}
+	}
+
+	return (0);
+}
+
 /* Scroll a line into the history. */
 void
-grid_scroll_line(struct grid_data *gd)
+grid_scroll_line(struct grid *gd)
 {
 	u_int	yy;
 
@@ -138,29 +183,37 @@ grid_scroll_line(struct grid_data *gd)
 	}
 
 	yy = gd->hsize + gd->sy;
-	gd->size = xrealloc(gd->size, yy + 1, sizeof *gd->size);
-	gd->data = xrealloc(gd->data, yy + 1, sizeof *gd->data);
 
-	gd->data[yy] = NULL;
+	gd->size = xrealloc(gd->size, yy + 1, sizeof *gd->size);
 	gd->size[yy] = 0;
+	gd->data = xrealloc(gd->data, yy + 1, sizeof *gd->data);
+	gd->data[yy] = NULL;
+
+	gd->usize = xrealloc(gd->usize, yy + 1, sizeof *gd->usize);
+	gd->usize[yy] = 0;
+	gd->udata = xrealloc(gd->udata, yy + 1, sizeof *gd->udata);
+	gd->udata[yy] = NULL;
 
 	gd->hsize++;
 }
 
 /* Reduce line to fit to cell. */
 void
-grid_reduce_line(struct grid_data *gd, u_int py, u_int sx)
+grid_reduce_line(struct grid *gd, u_int py, u_int sx)
 {
-	if (sx >= gd->size[py])
-		return;
-
-	gd->data[py] = xrealloc(gd->data[py], sx, sizeof **gd->data);
-	gd->size[py] = sx;
+	if (sx < gd->size[py]) {
+		gd->data[py] = xrealloc(gd->data[py], sx, sizeof **gd->data);
+		gd->size[py] = sx;
+	}
+	if (sx < gd->usize[py]) {
+		gd->udata[py] = xrealloc(gd->udata[py], sx, sizeof **gd->udata);
+		gd->usize[py] = sx;
+	}
 }
 
 /* Expand line to fit to cell. */
 void
-grid_expand_line(struct grid_data *gd, u_int py, u_int sx)
+grid_expand_line(struct grid *gd, u_int py, u_int sx)
 {
 	u_int	xx;
 
@@ -173,9 +226,20 @@ grid_expand_line(struct grid_data *gd, u_int py, u_int sx)
 	gd->size[py] = sx;
 }
 
+/* Expand line to fit to cell for UTF-8. */
+void
+grid_expand_line_utf8(struct grid *gd, u_int py, u_int sx)
+{
+	if (sx <= gd->usize[py])
+		return;
+
+	gd->udata[py] = xrealloc(gd->udata[py], sx, sizeof **gd->udata);
+	gd->usize[py] = sx;
+}
+
 /* Get cell for reading. */
 const struct grid_cell *
-grid_peek_cell(struct grid_data *gd, u_int px, u_int py)
+grid_peek_cell(struct grid *gd, u_int px, u_int py)
 {
 	if (grid_check_x(gd, px) != 0)
 		return (&grid_default_cell);
@@ -189,7 +253,7 @@ grid_peek_cell(struct grid_data *gd, u_int px, u_int py)
 
 /* Get cell at relative position (for writing). */
 struct grid_cell *
-grid_get_cell(struct grid_data *gd, u_int px, u_int py)
+grid_get_cell(struct grid *gd, u_int px, u_int py)
 {
 	if (grid_check_x(gd, px) != 0)
 		return (NULL);
@@ -203,7 +267,7 @@ grid_get_cell(struct grid_data *gd, u_int px, u_int py)
 /* Set cell at relative position. */
 void
 grid_set_cell(
-    struct grid_data *gd, u_int px, u_int py, const struct grid_cell *gc)
+    struct grid *gd, u_int px, u_int py, const struct grid_cell *gc)
 {
 	if (grid_check_x(gd, px) != 0)
 		return;
@@ -214,12 +278,53 @@ grid_set_cell(
 	grid_put_cell(gd, px, py, gc);
 }
 
+/* Get UTF-8 for reading. */
+const struct grid_utf8 *
+grid_peek_utf8(struct grid *gd, u_int px, u_int py)
+{
+	if (grid_check_x(gd, px) != 0)
+		return (NULL);
+	if (grid_check_y(gd, py) != 0)
+		return (NULL);
+
+	if (px >= gd->usize[py])
+		return (NULL);
+	return (&gd->udata[py][px]);
+}
+
+/* Get utf8 at relative position (for writing). */
+struct grid_utf8 *
+grid_get_utf8(struct grid *gd, u_int px, u_int py)
+{
+	if (grid_check_x(gd, px) != 0)
+		return (NULL);
+	if (grid_check_y(gd, py) != 0)
+		return (NULL);
+
+	grid_expand_line_utf8(gd, py, px + 1);
+	return (&gd->udata[py][px]);
+}
+
+/* Set utf8 at relative position. */
+void
+grid_set_utf8(
+    struct grid *gd, u_int px, u_int py, const struct grid_utf8 *gc)
+{
+	if (grid_check_x(gd, px) != 0)
+		return;
+	if (grid_check_y(gd, py) != 0)
+		return;
+
+	grid_expand_line_utf8(gd, py, px + 1);
+	grid_put_utf8(gd, px, py, gc);
+}
+
 /*
  * Clear area. Note this is different from a fill as it just omits unallocated
  * cells.
  */
 void
-grid_clear(struct grid_data *gd, u_int px, u_int py, u_int nx, u_int ny)
+grid_clear(struct grid *gd, u_int px, u_int py, u_int nx, u_int ny)
 {
 	u_int	xx, yy;
 
@@ -251,38 +356,9 @@ grid_clear(struct grid_data *gd, u_int px, u_int py, u_int nx, u_int ny)
 	}
 }
 
-/* Fill area. */
-void
-grid_fill(struct grid_data *gd,
-    const struct grid_cell *gc, u_int px, u_int py, u_int nx, u_int ny)
-{
-	u_int	xx, yy;
-
- 	GRID_DEBUG(gd, "px=%u, py=%u, nx=%u, ny=%u", px, py, nx, ny);
-
-	if (nx == 0 || ny == 0)
-		return;
-
-	if (grid_check_x(gd, px) != 0)
-		return;
-	if (grid_check_x(gd, px + nx - 1) != 0)
-		return;
-	if (grid_check_y(gd, py) != 0)
-		return;
-	if (grid_check_y(gd, py + ny - 1) != 0)
-		return;
-
-	for (yy = py; yy < py + ny; yy++) {
-		for (xx = px; xx < px + nx; xx++) {
-			grid_expand_line(gd, yy, xx + 1);
-			grid_put_cell(gd, xx, py, gc);
-		}
-	}
-}
-
 /* Clear lines. This just frees and truncates the lines. */
 void
-grid_clear_lines(struct grid_data *gd, u_int py, u_int ny)
+grid_clear_lines(struct grid *gd, u_int py, u_int ny)
 {
 	u_int	yy;
 
@@ -299,23 +375,20 @@ grid_clear_lines(struct grid_data *gd, u_int py, u_int ny)
 	for (yy = py; yy < py + ny; yy++) {
 		if (gd->data[yy] != NULL) {
 			xfree(gd->data[yy]);
-			gd->data[yy] = NULL;
+			gd->data[yy] = NULL;	
 			gd->size[yy] = 0;
 		}
+		if (gd->udata[yy] != NULL) {
+			xfree(gd->udata[yy]);
+			gd->udata[yy] = NULL;	
+			gd->usize[yy] = 0;
+		}
 	}
 }
 
-/* Fill a group of lines. */
-void
-grid_fill_lines(
-    struct grid_data *gd, const struct grid_cell *gc, u_int py, u_int ny)
-{
-	grid_fill(gd, gc, 0, py, gd->sx, ny);
-}
-
 /* Move a group of lines. */
 void
-grid_move_lines(struct grid_data *gd, u_int dy, u_int py, u_int ny)
+grid_move_lines(struct grid *gd, u_int dy, u_int py, u_int ny)
 {
 	u_int	yy;
 
@@ -340,21 +413,26 @@ grid_move_lines(struct grid_data *gd, u_int dy, u_int py, u_int ny)
 		grid_clear_lines(gd, yy, 1);
 	}
 
-	memmove(&gd->data[dy], &gd->data[py], ny * (sizeof *gd->data));
+	memmove(&gd->data[dy], &gd->data[py], ny * (sizeof *gd->data));	
 	memmove(&gd->size[dy], &gd->size[py], ny * (sizeof *gd->size));
 
+	memmove(&gd->udata[dy], &gd->udata[py], ny * (sizeof *gd->udata));	
+	memmove(&gd->usize[dy], &gd->usize[py], ny * (sizeof *gd->usize));
+
 	/* Wipe any lines that have been moved (without freeing them). */
 	for (yy = py; yy < py + ny; yy++) {
 		if (yy >= dy && yy < dy + ny)
 			continue;
 		gd->data[yy] = NULL;
 		gd->size[yy] = 0;
+		gd->udata[yy] = NULL;
+		gd->usize[yy] = 0;
 	}
 }
 
 /* Clear a group of cells. */
 void
-grid_clear_cells(struct grid_data *gd, u_int px, u_int py, u_int nx)
+grid_clear_cells(struct grid *gd, u_int px, u_int py, u_int nx)
 {
 	u_int	xx;
 
@@ -379,7 +457,7 @@ grid_clear_cells(struct grid_data *gd, u_int px, u_int py, u_int nx)
 
 /* Move a group of cells. */
 void
-grid_move_cells(struct grid_data *gd, u_int dx, u_int px, u_int py, u_int nx)
+grid_move_cells(struct grid *gd, u_int dx, u_int px, u_int py, u_int nx)
 {
 	u_int	xx;
 
@@ -401,12 +479,19 @@ grid_move_cells(struct grid_data *gd, u_int dx, u_int px, u_int py, u_int nx)
 	grid_expand_line(gd, py, dx + nx);
 	memmove(&gd->data[py][dx], &gd->data[py][px], nx * (sizeof **gd->data));
 
+	if (gd->udata[py] != NULL) {
+		grid_expand_line_utf8(gd, py, px + nx);
+		grid_expand_line_utf8(gd, py, dx + nx);
+		memmove(&gd->udata[py][dx],
+		    &gd->udata[py][px], nx * (sizeof **gd->udata));
+	}
+
 	/* Wipe any cells that have been moved. */
 	for (xx = px; xx < px + nx; xx++) {
 		if (xx >= dx && xx < dx + nx)
 			continue;
-		memcpy(
-		    &gd->data[py][xx], &grid_default_cell, sizeof **gd->data);
+		grid_put_cell(gd, xx, py, &grid_default_cell);
 	}
 }
 
+
