@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.10 2008/12/08 16:19:51 nicm Exp $ */
+/* $Id: log.c,v 1.11 2009/03/31 21:22:10 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -111,7 +111,11 @@ log_vwrite(int pri, const char *msg, va_list ap)
 
 	switch (log_type) {
 	case LOG_TYPE_SYSLOG:
+#ifdef NO_VSYSLOG
+		/* XXX */
+#else
 		vsyslog(pri, msg, ap);
+#endif
 		break;
 	case LOG_TYPE_TTY:
 		if (pri == LOG_INFO)
