@@ -1,4 +1,4 @@
-/* $Id: names.c,v 1.2 2009/01/26 22:57:19 nicm Exp $ */
+/* $Id: names.c,v 1.4 2009/02/13 00:43:04 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -54,13 +54,14 @@ set_window_names(void)
 		if (w->active->screen != &w->active->base)
 			name = NULL;
 		else
-			name = get_argv0(w->active->fd, w->active->tty);
+			name = osdep_get_name(w->active->fd, w->active->tty);
 		if (name == NULL)
 			wname = default_window_name(w);
 		else {
 			wname = parse_window_name(name);
 			xfree(name);
 		}
+
 		if (strcmp(wname, w->name) == 0)
 			xfree(wname);
 		else {
