@@ -1,4 +1,4 @@
-/* $Id: osdep-darwin.c,v 1.4 2009/02/07 19:41:35 nicm Exp $ */
+/* $Id: osdep-darwin.c,v 1.9 2009/02/13 00:43:04 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Joshua Elsasser <josh@elsasser.org>
@@ -25,16 +25,12 @@
 #include <string.h>
 #include <unistd.h>
 
-char	*get_argv0(int, char *);
+char	*osdep_get_name(int, char *);
 
-/*
- * XXX This actually returns the executable path, not the process's argv[0].
- * Anyone who wishes to complain about this is welcome to grab a copy of
- * Apple's 'ps' source and start digging.
- */
+#define unused __attribute__ ((unused))
 
 char *
-get_argv0(int fd, __attribute__ ((unused)) char *tty)
+osdep_get_name(int fd, unused char *tty)
 {
 	int	mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, 0 };
         size_t	size;
@@ -44,8 +40,9 @@ get_argv0(int fd, __attribute__ ((unused)) char *tty)
 		return (NULL);
 
 	size = sizeof kp;
-	if (sysctl(mib, 4, &kp, &size, NULL, 0) == -1 ||
-	    kp.kp_proc.p_comm[0] == '\0')
+	if (sysctl(mib, 4, &kp, &size, NULL, 0) == -1)
+		return (NULL);
+	if (*kp.kp_proc.p_comm == '\0')
 		return (NULL);
 
 	return (strdup(kp.kp_proc.p_comm));
