@@ -1,4 +1,4 @@
-/* $Id: osdep-unknown.c,v 1.2 2009/01/20 22:17:53 nicm Exp $ */
+/* $Id: osdep-unknown.c,v 1.4 2009/03/31 21:22:10 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,14 +17,14 @@
  */
 
 #if !defined(__OpenBSD__) && !defined(__FreeBSD__) && !defined(__linux__) && \
-	!defined(__APPLE__)
+	!defined(__APPLE__) && !defined(__NetBSD__)
 
 #include <sys/types.h>
 
 #include "tmux.h"
 
 char *
-get_argv0(unused pid_t pgrp)
+osdep_get_name(unused int fd, unused char *tty)
 {
 	return (NULL);
 }
