@@ -1,4 +1,4 @@
-/* $Id: resize.c,v 1.18 2009/01/11 23:31:46 nicm Exp $ */
+/* $Id: resize.c,v 1.21 2009/04/01 21:10:08 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -45,11 +45,11 @@
 void
 recalculate_sizes(void)
 {
-	struct session	*s;
-	struct client	*c;
-	struct window	*w;
-	u_int		 i, j, ssx, ssy, has, limit;
-	int		 flag;
+	struct session		*s;
+	struct client		*c;
+	struct window		*w;
+	u_int		 	 i, j, ssx, ssy, has, limit;
+	int		 	 flag;
 
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
@@ -62,10 +62,10 @@ recalculate_sizes(void)
 			if (c == NULL)
 				continue;
 			if (c->session == s) {
-				if (c->sx < ssx)
-					ssx = c->sx;
-				if (c->sy < ssy)
-					ssy = c->sy;
+				if (c->tty.sx < ssx)
+					ssx = c->tty.sx;
+				if (c->tty.sy < ssy)
+					ssy = c->tty.sy;
 			}
 		}
 		if (ssx == UINT_MAX || ssy == UINT_MAX) {
@@ -133,5 +133,6 @@ recalculate_sizes(void)
 
 		window_resize(w, ssx, ssy);
 		server_redraw_window(w);
+		layout_refresh(w, 0);
 	}
 }
