@@ -1,4 +1,4 @@
-/* $Id: screen-write.c,v 1.31 2009/01/28 19:52:21 nicm Exp $ */
+/* $Id: screen-write.c,v 1.43 2009/03/30 20:14:50 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -22,22 +22,19 @@
 
 #include "tmux.h"
 
+void	screen_write_save(struct screen_write_ctx *);
+void	screen_write_overwrite(struct screen_write_ctx *);
+
 /* Initialise writing with a window. */
 void
 screen_write_start(
     struct screen_write_ctx *ctx, struct window_pane *wp, struct screen *s)
 {
-	ctx->s = s;
-	if (wp != NULL) {
-		ctx->write = tty_write_window;
-		ctx->data = wp;
-		if (ctx->s == NULL)
-			ctx->s = wp->screen;
-		tty_write_update_mode(ctx->data, ctx->s->mode & ~MODE_CURSOR);
-	} else {
-		ctx->write = NULL;
-		ctx->data = NULL;
-	}
+	ctx->wp = wp;
+	if (wp != NULL && s == NULL)
+		ctx->s = wp->screen;
+	else
+		ctx->s = s;
 }
 
 /* Finish writing. */
@@ -52,7 +49,7 @@ screen_write_putc(
     struct screen_write_ctx *ctx, struct grid_cell *gc, u_char ch)
 {
 	gc->data = ch;
-	screen_write_cell(ctx, gc);
+	screen_write_cell(ctx, gc, NULL);
 }
 
 /* Write string. */
@@ -79,8 +76,10 @@ screen_write_copy(struct screen_write_ctx *ctx,
     struct screen *src, u_int px, u_int py, u_int nx, u_int ny)
 {
 	struct screen		*s = ctx->s;
-	struct grid_data	*gd = src->grid;
+	struct grid		*gd = src->grid;
 	const struct grid_cell	*gc;
+	struct grid_utf8	*gu;
+	u_char			*udata;
 	u_int		 	 xx, yy, cx, cy;
 
 	cx = s->cx;
@@ -91,13 +90,33 @@ screen_write_copy(struct screen_write_ctx *ctx,
 				gc = &grid_default_cell;
 			else
 				gc = grid_peek_cell(gd, xx, yy);
-			screen_write_cell(ctx, gc);
+
+			udata = NULL;
+			if (gc->flags & GRID_FLAG_UTF8) {
+				gu = grid_get_utf8(gd, xx, yy);
+				udata = gu->data;
+			}
+
+			screen_write_cell(ctx, gc, udata);
 		}
 		cy++;
 		screen_write_cursormove(ctx, cx, cy);
 	}
 }
 
+/* Save cursor and region positions. */
+void
+screen_write_save(struct screen_write_ctx *ctx)
+{
+	struct screen	*s = ctx->s;
+
+	s->old_cx = s->cx;
+	s->old_cy = s->cy;
+
+	s->old_rlower = s->rlower;
+	s->old_rupper = s->rupper;
+}
+
 /* Cursor up by ny. */
 void
 screen_write_cursorup(struct screen_write_ctx *ctx, u_int ny)
@@ -180,11 +199,12 @@ screen_write_insertcharacter(struct screen_write_ctx *ctx, u_int nx)
 	if (nx == 0)
 		return;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_INSERTCHARACTER, nx);
+	screen_write_save(ctx);
 
 	if (s->cx <= screen_size_x(s) - 1)
 		grid_view_insert_cells(s->grid, s->cx, s->cy, nx);
+
+	tty_write_cmd(ctx->wp, TTY_INSERTCHARACTER, nx);
 }
 
 /* Delete nx characters. */
@@ -201,11 +221,12 @@ screen_write_deletecharacter(struct screen_write_ctx *ctx, u_int nx)
 	if (nx == 0)
 		return;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_DELETECHARACTER, nx);
+	screen_write_save(ctx);
 
 	if (s->cx <= screen_size_x(s) - 1)
 		grid_view_delete_cells(s->grid, s->cx, s->cy, nx);
+
+	tty_write_cmd(ctx->wp, TTY_DELETECHARACTER, nx);
 }
 
 /* Insert ny lines. */
@@ -222,8 +243,7 @@ screen_write_insertline(struct screen_write_ctx *ctx, u_int ny)
 	if (ny == 0)
 		return;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_INSERTLINE, ny);
+	screen_write_save(ctx);
 
 	if (s->cy < s->rupper || s->cy > s->rlower)
 		grid_view_insert_lines(s->grid, s->cy, ny);
@@ -231,6 +251,8 @@ screen_write_insertline(struct screen_write_ctx *ctx, u_int ny)
 		grid_view_insert_lines_region(
 		    s->grid, s->rupper, s->rlower, s->cy, ny);
 	}
+
+	tty_write_cmd(ctx->wp, TTY_INSERTLINE, ny);
 }
 
 /* Delete ny lines. */
@@ -247,8 +269,7 @@ screen_write_deleteline(struct screen_write_ctx *ctx, u_int ny)
 	if (ny == 0)
 		return;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_DELETELINE, ny);
+	screen_write_save(ctx);
 
 	if (s->cy < s->rupper || s->cy > s->rlower)
 		grid_view_delete_lines(s->grid, s->cy, ny);
@@ -256,6 +277,8 @@ screen_write_deleteline(struct screen_write_ctx *ctx, u_int ny)
 		grid_view_delete_lines_region(
 		    s->grid, s->rupper, s->rlower, s->cy, ny);
 	}
+
+	tty_write_cmd(ctx->wp, TTY_DELETELINE, ny);
 }
 
 /* Clear line at cursor. */
@@ -264,10 +287,11 @@ screen_write_clearline(struct screen_write_ctx *ctx)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARLINE);
+	screen_write_save(ctx);
 
 	grid_view_clear(s->grid, 0, s->cy, screen_size_x(s), 1);
+
+	tty_write_cmd(ctx->wp, TTY_CLEARLINE);
 }
 
 /* Clear to end of line from cursor. */
@@ -277,13 +301,14 @@ screen_write_clearendofline(struct screen_write_ctx *ctx)
 	struct screen	*s = ctx->s;
 	u_int		 sx;
 
-	sx = screen_size_x(s);
+	screen_write_save(ctx);
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARENDOFLINE);
+	sx = screen_size_x(s);
 
 	if (s->cx <= sx - 1)
 		grid_view_clear(s->grid, s->cx, s->cy, sx - s->cx, 1);
+
+ 	tty_write_cmd(ctx->wp, TTY_CLEARENDOFLINE);
 }
 
 /* Clear to start of line from cursor. */
@@ -293,15 +318,16 @@ screen_write_clearstartofline(struct screen_write_ctx *ctx)
 	struct screen	*s = ctx->s;
 	u_int		 sx;
 
-	sx = screen_size_x(s);
+	screen_write_save(ctx);
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARSTARTOFLINE);
+	sx = screen_size_x(s);
 
 	if (s->cx > sx - 1)
 		grid_view_clear(s->grid, 0, s->cy, sx, 1);
 	else
 		grid_view_clear(s->grid, 0, s->cy, s->cx + 1, 1);
+
+	tty_write_cmd(ctx->wp, TTY_CLEARSTARTOFLINE);
 }
 
 /* Move cursor to px,py.  */
@@ -337,13 +363,14 @@ screen_write_reverseindex(struct screen_write_ctx *ctx)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_REVERSEINDEX);
+	screen_write_save(ctx);
 
 	if (s->cy == s->rupper)
 		grid_view_scroll_region_down(s->grid, s->rupper, s->rlower);
 	else if (s->cy > 0)
 		s->cy--;
+
+	tty_write_cmd(ctx->wp, TTY_REVERSEINDEX);
 }
 
 /* Set scroll region. */
@@ -362,7 +389,7 @@ screen_write_scrollregion(
 
 	/* Cursor moves to top-left. */
 	s->cx = 0;
-	s->cy = rupper;
+	s->cy = 0;
 
 	s->rupper = rupper;
 	s->rlower = rlower;
@@ -398,13 +425,14 @@ screen_write_linefeed(struct screen_write_ctx *ctx)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_LINEFEED);
+	screen_write_save(ctx);
 
 	if (s->cy == s->rlower)
 		grid_view_scroll_region_up(s->grid, s->rupper, s->rlower);
 	else if (s->cy < screen_size_y(s) - 1)
 		s->cy++;
+
+ 	tty_write_cmd(ctx->wp, TTY_LINEFEED);
 }
 
 /* Carriage return (cursor to start of line). */
@@ -447,15 +475,16 @@ screen_write_clearendofscreen(struct screen_write_ctx *ctx)
 	struct screen	*s = ctx->s;
 	u_int		 sx, sy;
 
+	screen_write_save(ctx);
+
 	sx = screen_size_x(s);
 	sy = screen_size_y(s);
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARENDOFSCREEN);
-
 	if (s->cx <= sx - 1)
 		grid_view_clear(s->grid, s->cx, s->cy, sx - s->cx, 1);
 	grid_view_clear(s->grid, 0, s->cy + 1, sx, sy - (s->cy + 1));
+
+	tty_write_cmd(ctx->wp, TTY_CLEARENDOFSCREEN);
 }
 
 /* Clear to start of screen. */
@@ -465,10 +494,9 @@ screen_write_clearstartofscreen(struct screen_write_ctx *ctx)
 	struct screen	*s = ctx->s;
 	u_int		 sx;
 
-	sx = screen_size_x(s);
+	screen_write_save(ctx);
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARSTARTOFSCREEN);
+	sx = screen_size_x(s);
 
 	if (s->cy > 0)
 		grid_view_clear(s->grid, 0, 0, sx, s->cy - 1);
@@ -476,6 +504,8 @@ screen_write_clearstartofscreen(struct screen_write_ctx *ctx)
 		grid_view_clear(s->grid, 0, s->cy, sx, 1);
 	else
 		grid_view_clear(s->grid, 0, s->cy, s->cx, 1);
+
+	tty_write_cmd(ctx->wp, TTY_CLEARSTARTOFSCREEN);
 }
 
 /* Clear entire screen. */
@@ -484,39 +514,74 @@ screen_write_clearscreen(struct screen_write_ctx *ctx)
 {
 	struct screen	*s = ctx->s;
 
-	if (ctx->write != NULL)
-		ctx->write(ctx->data, TTY_CLEARSCREEN);
+	screen_write_save(ctx);
 
 	grid_view_clear(s->grid, 0, 0, screen_size_x(s), screen_size_y(s));
+
+	tty_write_cmd(ctx->wp, TTY_CLEARSCREEN);
 }
 
 /* Write cell data. */
 void
-screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
+screen_write_cell(
+    struct screen_write_ctx *ctx, const struct grid_cell *gc, u_char *udata)
 {
 	struct screen		*s = ctx->s;
-	struct grid_data	*gd = s->grid;
-	u_int		 	 width, xx;
-	const struct grid_cell 	*hc;
-	struct grid_cell 	*ic, tc;
+	struct grid		*gd = s->grid;
+	struct grid_utf8	 gu, *tmp_gu;
+	u_int		 	 width, uvalue, xx, i;
+	struct grid_cell 	 tmp_gc, *tmp_gc2;
+	size_t			 size;
+
+	/* Ignore padding. */
+	if (gc->flags & GRID_FLAG_PADDING)
+		return;
 
 	/* Find character width. */
-	if (gc->flags & GRID_FLAG_UTF8)
-		width = utf8_width(gc->data);
-	else
+	if (gc->flags & GRID_FLAG_UTF8) {
+		uvalue = utf8_combine(udata);
+		width = utf8_width(uvalue);
+
+		gu.width = width;
+		memcpy(&gu.data, udata, sizeof gu.data);
+ 	} else
 		width = 1;
 
-	/* Discard zero-width characters. */
-	if (width == 0)
+	/* If the width is zero, combine onto the previous character. */
+	if (width == 0) {
+		if (s->cx == 0)
+			return;
+		tmp_gc2 = grid_view_get_cell(gd, s->cx - 1, s->cy);
+		if (!(tmp_gc2->flags & GRID_FLAG_UTF8)) {
+			tmp_gc2->flags |= GRID_FLAG_UTF8;
+			memset(&gu.data, 0xff, sizeof gu.data);
+			*gu.data = tmp_gc2->data;
+			gu.width = 1;
+			grid_view_set_utf8(gd, s->cx - 1, s->cy, &gu);
+		}
+		tmp_gu = grid_view_get_utf8(gd, s->cx - 1, s->cy);
+
+		for (i = 0; i < UTF8_SIZE; i++) {
+			if (tmp_gu->data[i] == 0xff)
+				break;
+		}
+		memcpy(tmp_gu->data + i, udata, UTF8_SIZE - i);
+
+		/* Assume the previous character has just been input. */
+		for (size = 0; size < UTF8_SIZE; size++) {
+			if (udata[size] == 0xff)
+				break;
+		}
+		tty_write_cmd(ctx->wp, TTY_RAW, udata, size);
 		return;
+	}
 
 	/* If the character is wider than the screen, don't print it. */
 	if (width > screen_size_x(s)) {
-		memcpy(&tc, gc, sizeof tc);
-		tc.data = '_';
-
+		memcpy(&tmp_gc, gc, sizeof tmp_gc);
+		tmp_gc.data = '_';
 		width = 1;
-		gc = &tc;
+		gc = &tmp_gc;
 	}
 
 	/* Check this will fit on the current line; scroll if not. */
@@ -529,17 +594,58 @@ screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
 	if (s->cx > screen_size_x(s) - 1 || s->cy > screen_size_y(s) - 1)
 		return;
 
+	/* Handle overwriting of UTF-8 characters. */
+	screen_write_overwrite(ctx);
+
 	/*
-	 * UTF-8 wide characters are a bit of an annoyance. They take up more
-	 * than one cell on the screen, so following cells must not be drawn by
-	 * marking them as padding.
-	 *
-	 * So far, so good. The problem is, when overwriting a padding cell, or
-	 * a UTF-8 character, it is necessary to also overwrite any other cells
-	 * which covered by the same character.
+	 * If the new character is UTF-8 wide, fill in padding cells. Have
+	 * already ensured there is enough room.
 	 */
-	hc = grid_view_peek_cell(gd, s->cx, s->cy);
-	if (hc->flags & GRID_FLAG_PADDING) {
+	for (xx = s->cx + 1; xx < s->cx + width; xx++) {
+		tmp_gc2 = grid_view_get_cell(gd, xx, s->cy);
+		if (tmp_gc2 != NULL)
+			tmp_gc2->flags |= GRID_FLAG_PADDING;
+	}
+
+	/* Set the cell. */
+	grid_view_set_cell(gd, s->cx, s->cy, gc);
+	if (gc->flags & GRID_FLAG_UTF8)
+		grid_view_set_utf8(gd, s->cx, s->cy, &gu);
+	
+	/* Move the cursor. */
+	screen_write_save(ctx);
+	s->cx += width;
+
+	/* Draw to the screen if necessary. */
+	if (screen_check_selection(s, s->cx - width, s->cy)) {
+		s->sel.cell.data = gc->data;
+		tty_write_cmd(ctx->wp, TTY_CELL, &s->sel.cell, &gu);
+	} else
+		tty_write_cmd(ctx->wp, TTY_CELL, gc, &gu);
+}
+
+/*
+ * UTF-8 wide characters are a bit of an annoyance. They take up more than one
+ * cell on the screen, so following cells must not be drawn by marking them as
+ * padding.
+ *
+ * So far, so good. The problem is, when overwriting a padding cell, or a UTF-8
+ * character, it is necessary to also overwrite any other cells which covered
+ * by the same character.
+ */
+void
+screen_write_overwrite(struct screen_write_ctx *ctx)
+{
+	struct screen		*s = ctx->s;
+	struct grid		*gd = s->grid;
+	const struct grid_cell	*gc;
+	const struct grid_utf8	*gu;
+	u_int			 xx;
+
+	gc = grid_view_peek_cell(gd, s->cx, s->cy);
+	gu = grid_view_peek_utf8(gd, s->cx, s->cy);
+
+	if (gc->flags & GRID_FLAG_PADDING) {
 		/*
 		 * A padding cell, so clear any following and leading padding
 		 * cells back to the character. Don't overwrite the current
@@ -547,8 +653,8 @@ screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
 		 */
 		xx = s->cx + 1;
 		while (--xx > 0) {
-			hc = grid_view_peek_cell(gd, xx, s->cy);
-			if (!(hc->flags & GRID_FLAG_PADDING))
+			gc = grid_view_peek_cell(gd, xx, s->cy);
+			if (!(gc->flags & GRID_FLAG_PADDING))
 				break;
 			grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 		}
@@ -559,46 +665,21 @@ screen_write_cell(struct screen_write_ctx *ctx, const struct grid_cell *gc)
 		/* Overwrite following padding cells. */
 		xx = s->cx;
 		while (++xx < screen_size_x(s)) {
-			hc = grid_view_peek_cell(gd, xx, s->cy);
-			if (!(hc->flags & GRID_FLAG_PADDING))
+			gc = grid_view_peek_cell(gd, xx, s->cy);
+			if (!(gc->flags & GRID_FLAG_PADDING))
 				break;
 			grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 		}
-	} else if (hc->flags & GRID_FLAG_UTF8 && utf8_width(hc->data) > 1) {
+	} else if (gc->flags & GRID_FLAG_UTF8 && gu->width > 1) {
 		/*
 		 * An UTF-8 wide cell; overwrite following padding cells only.
 		 */
 		xx = s->cx;
 		while (++xx < screen_size_x(s)) {
-			hc = grid_view_peek_cell(gd, xx, s->cy);
-			if (!(hc->flags & GRID_FLAG_PADDING))
+			gc = grid_view_peek_cell(gd, xx, s->cy);
+			if (!(gc->flags & GRID_FLAG_PADDING))
 				break;
 			grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 		}
 	}
-
-	/*
-	 * If the new character is UTF-8 wide, fill in padding cells. Have
-	 * already ensured there is enough room.
-	 */
-	for (xx = s->cx + 1; xx < s->cx + width; xx++) {
-		ic = grid_view_get_cell(gd, xx, s->cy);
-		if (ic != NULL)
-			ic->flags |= GRID_FLAG_PADDING;
-	}
-
-	/* Write the actual cell. */
-	grid_view_set_cell(gd, s->cx, s->cy, gc);
-
-	if (ctx->write != NULL) {
-		if (screen_check_selection(s, s->cx, s->cy)) {
-			memcpy(&tc, &s->sel.cell, sizeof tc);
-			tc.data = gc->data;
-			ctx->write(ctx->data, TTY_CELL, &tc);
-		} else
-			ctx->write(ctx->data, TTY_CELL, gc);
-	}
-
-	/* Move the cursor. */
-	s->cx += width;
 }
