@@ -1,4 +1,4 @@
-/* $Id: server-msg.c,v 1.62 2009/01/28 17:57:00 nicm Exp $ */
+/* $Id: server-msg.c,v 1.65 2009/03/07 10:29:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -192,19 +192,21 @@ server_msg_fn_identify(struct hdr *hdr, struct client *c)
 		return (0);
 	}
 
-	c->sx = data.sx;
-	c->sy = data.sy;
+	c->tty.sx = data.sx;
+	c->tty.sy = data.sy;
 
 	c->cwd = NULL;
 	if (*data.cwd != '\0')
 		c->cwd = xstrdup(data.cwd);
 
 	data.tty[(sizeof data.tty) - 1] = '\0';
-	tty_init(&c->tty, data.tty, xstrdup(term));
+	tty_init(&c->tty, data.tty, term);
 	if (data.flags & IDENTIFY_UTF8)
 		c->tty.flags |= TTY_UTF8;
 	if (data.flags & IDENTIFY_256COLOURS)
 		c->tty.term_flags |= TERM_256COLOURS;
+	else if (data.flags & IDENTIFY_88COLOURS)
+		c->tty.term_flags |= TERM_88COLOURS;
 	if (data.flags & IDENTIFY_HASDEFAULTS)
 		c->tty.term_flags |= TERM_HASDEFAULTS;
 	xfree(term);
@@ -225,12 +227,12 @@ server_msg_fn_resize(struct hdr *hdr, struct client *c)
 
 	log_debug("resize msg from client: %u,%u", data.sx, data.sy);
 
-	c->sx = data.sx;
-	if (c->sx == 0)
-		c->sx = 80;
-	c->sy = data.sy;
-	if (c->sy == 0)
-		c->sy = 25;
+	c->tty.sx = data.sx;
+	if (c->tty.sx == 0)
+		c->tty.sx = 80;
+	c->tty.sy = data.sy;
+	if (c->tty.sy == 0)
+		c->tty.sy = 25;
 	
 	c->tty.cx = UINT_MAX;
 	c->tty.cy = UINT_MAX;
