@@ -1,4 +1,4 @@
-/* $Id: server.c,v 1.116 2009/01/29 20:13:12 nicm Exp $ */
+/* $Id: server.c,v 1.137 2009/04/16 10:59:59 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -21,6 +21,7 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
+#include <sys/wait.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -42,8 +43,9 @@
 /* Client list. */
 struct clients	 clients;
 
-int		 server_main(const char *, int);
+int		 server_main(int);
 void		 server_shutdown(void);
+void		 server_child_signal(void);
 void		 server_fill_windows(struct pollfd **);
 void		 server_handle_windows(struct pollfd **);
 void		 server_fill_clients(struct pollfd **);
@@ -57,7 +59,7 @@ void		 server_check_redraw(struct client *);
 void		 server_redraw_locked(struct client *);
 void		 server_check_timers(struct client *);
 void		 server_second_timers(void);
-int		 server_update_socket(const char *);
+int		 server_update_socket(void);
 
 /* Create a new client. */
 struct client *
@@ -71,6 +73,8 @@ server_create_client(int fd)
 		fatal("fcntl failed");
 	if (fcntl(fd, F_SETFL, mode|O_NONBLOCK) == -1)
 		fatal("fcntl failed");
+	if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
+		fatal("fcntl failed");
 
 	c = xcalloc(1, sizeof *c);
 	c->fd = fd;
@@ -83,9 +87,9 @@ server_create_client(int fd)
 	c->title = NULL;
 
 	c->session = NULL;
-	c->sx = 80;
-	c->sy = 25;
-	screen_init(&c->status, c->sx, 1, 0);
+	c->tty.sx = 80;
+	c->tty.sy = 25;
+	screen_init(&c->status, c->tty.sx, 1, 0);
 
 	c->message_string = NULL;
 
@@ -118,13 +122,16 @@ server_client_index(struct client *c)
 
 /* Fork new server. */
 int
-server_start(const char *path)
+server_start(char *path)
 {
 	struct sockaddr_un	sa;
 	size_t			size;
 	mode_t			mask;
 	int		   	n, fd, pair[2], mode;
 	char		       *cause;
+#ifndef NO_SETPROCTITLE
+	char			rpathbuf[MAXPATHLEN];
+#endif
 
 	/* The first client is special and gets a socketpair; create it. */
 	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pair) != 0)
@@ -149,7 +156,7 @@ server_start(const char *path)
 	 * Must daemonise before loading configuration as the PID changes so
 	 * $TMUX would be wrong for sessions created in the config file.
 	 */
-	if (daemon(1, 0) != 0)
+	if (daemon(1, 1) != 0)
 		fatal("daemon failed");
 
 	ARRAY_INIT(&windows);
@@ -161,22 +168,27 @@ server_start(const char *path)
 	server_password = NULL;
 	server_activity = time(NULL);
 
+	start_time = time(NULL);
+	socket_path = path;
+
 	if (cfg_file != NULL && load_cfg(cfg_file, &cause) != 0) {
 		log_warnx("%s", cause);
 		exit(1);
 	}
-
 	logfile("server");
-#ifndef NO_SETPROCTITLE
-	setproctitle("server (%s)", path);
-#endif
+
 	log_debug("server started, pid %ld", (long) getpid());
-	start_time = time(NULL);
-	socket_path = path;
+	log_debug("socket path %s", socket_path);
+
+#ifndef NO_SETPROCTITLE
+	if (realpath(socket_path, rpathbuf) == NULL)
+		strlcpy(rpathbuf, socket_path, sizeof rpathbuf);
+	setproctitle("server (%s)", rpathbuf);
+#endif
 	
 	memset(&sa, 0, sizeof sa);
 	sa.sun_family = AF_UNIX;
-	size = strlcpy(sa.sun_path, path, sizeof sa.sun_path);
+	size = strlcpy(sa.sun_path, socket_path, sizeof sa.sun_path);
 	if (size >= sizeof sa.sun_path) {
 		errno = ENAMETOOLONG;
 		fatal("socket failed");
@@ -203,7 +215,7 @@ server_start(const char *path)
 
 	server_create_client(pair[1]);
 
-	n = server_main(path, fd);
+	n = server_main(fd);
 #ifdef DEBUG
 	xmalloc_report(getpid(), "server");
 #endif
@@ -212,7 +224,7 @@ server_start(const char *path)
 
 /* Main server loop. */
 int
-server_main(const char *srv_path, int srv_fd)
+server_main(int srv_fd)
 {
 	struct window	*w;
 	struct pollfd	*pfds, *pfd;
@@ -230,6 +242,12 @@ server_main(const char *srv_path, int srv_fd)
 		if (sigterm)
 			server_shutdown();
 
+		/* Handle child exit. */
+		if (sigchld) {
+			server_child_signal();
+			sigchld = 0;
+		}
+
 		/* Initialise pollfd array. */
 		nfds = 1;
 		for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
@@ -239,6 +257,7 @@ server_main(const char *srv_path, int srv_fd)
 		}
 		nfds += ARRAY_LENGTH(&clients) * 2;
 		pfds = xrealloc(pfds, nfds, sizeof *pfds);
+		memset(pfds, 0, nfds * sizeof *pfds);
 		pfd = pfds;
 
 		/* Fill server socket. */
@@ -252,7 +271,7 @@ server_main(const char *srv_path, int srv_fd)
 
 		/* Update socket permissions. */
 		xtimeout = INFTIM;
-		if (sigterm || server_update_socket(srv_path) != 0)
+		if (sigterm || server_update_socket() != 0)
 			xtimeout = 100;
 
 		/* Do the poll. */
@@ -326,7 +345,14 @@ server_main(const char *srv_path, int srv_fd)
 	key_bindings_free();
 
 	close(srv_fd);
-	unlink(srv_path);
+
+	unlink(socket_path);
+	xfree(socket_path);
+
+	options_free(&global_options);
+	options_free(&global_window_options);
+	if (server_password != NULL)
+		xfree(server_password);
 
 	return (0);
 }
@@ -358,6 +384,44 @@ server_shutdown(void)
 			server_write_client(c, MSG_SHUTDOWN, NULL, 0);
 	}
 }
+
+/* Handle SIGCHLD. */
+void
+server_child_signal(void)
+{
+	struct window		*w;
+	struct window_pane	*wp;
+	int		 	 status;
+	pid_t		 	 pid;
+	u_int		 	 i;
+
+	for (;;) {
+		switch (pid = waitpid(WAIT_ANY, &status, WNOHANG|WUNTRACED)) {
+		case -1:
+			if (errno == ECHILD)
+				return;
+			fatal("waitpid");
+		case 0:
+			return;
+		}
+		if (!WIFSTOPPED(status))
+			continue;
+		if (WSTOPSIG(status) == SIGTTIN || WSTOPSIG(status) == SIGTTOU)
+			continue;
+
+		for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
+			w = ARRAY_ITEM(&windows, i);
+			if (w == NULL)
+				continue;
+			TAILQ_FOREACH(wp, &w->panes, entry) {
+				if (wp->pid == pid) {
+					if (killpg(pid, SIGCONT) != 0)
+						kill(pid, SIGCONT);
+				}
+			}
+		}
+	}
+}
 	
 /* Fill window pollfds. */
 void
@@ -416,9 +480,10 @@ server_handle_windows(struct pollfd **pfd)
 void
 server_check_redraw(struct client *c)
 {
-	struct session	*s;
-	char		 title[512];
-	int		 flags;
+	struct session		*s;
+	struct window_pane	*wp;
+	char		 	 title[512];
+	int		 	 flags, redraw;
 
 	if (c == NULL || c->session == NULL)
 		return;
@@ -441,18 +506,26 @@ server_check_redraw(struct client *c)
 
 	if (c->flags & (CLIENT_REDRAW|CLIENT_STATUS)) {
 		if (c->message_string != NULL)
-			status_message_redraw(c);
+			redraw = status_message_redraw(c);
 		else if (c->prompt_string != NULL)
-			status_prompt_redraw(c);
+			redraw = status_prompt_redraw(c);
 		else
-			status_redraw(c);
+			redraw = status_redraw(c);
+		if (!redraw)
+			c->flags &= ~CLIENT_STATUS;
 	}
 
 	if (c->flags & CLIENT_REDRAW) {
 		if (server_locked)
 			server_redraw_locked(c);
 		else
- 			screen_redraw_screen(c, NULL);
+ 			screen_redraw_screen(c);
+		c->flags &= ~CLIENT_STATUS;
+	} else {
+		TAILQ_FOREACH(wp, &c->session->curw->window->panes, entry) {
+			if (wp->flags & PANE_REDRAW)
+				screen_redraw_pane(c, wp);
+		}
 	}
 
 	if (c->flags & CLIENT_STATUS)
@@ -469,11 +542,11 @@ server_redraw_locked(struct client *c)
 {
 	struct screen_write_ctx	ctx;
 	struct screen		screen;
-	u_int			colour, xx, yy;
+	u_int			colour, xx, yy, i;
 	int    			style;
 
-	xx = c->sx;
-	yy = c->sy - 1;
+	xx = c->tty.sx;
+	yy = c->tty.sy - 1;
 	if (xx == 0 || yy == 0)
 		return;
 	colour = options_get_number(
@@ -487,7 +560,9 @@ server_redraw_locked(struct client *c)
 	clock_draw(&ctx, colour, style);
 	screen_write_stop(&ctx);
 
-	screen_redraw_screen(c, &screen);
+	for (i = 0; i < screen_size_y(&screen); i++)
+		tty_draw_line(&c->tty, &screen, i, 0, 0);
+	screen_redraw_status(c);
 
 	screen_free(&screen);
 }
@@ -508,7 +583,7 @@ server_check_timers(struct client *c)
 		fatal("gettimeofday");
 
 	if (c->message_string != NULL && timercmp(&tv, &c->message_timer, >))
-		server_clear_client_message(c);
+		status_message_clear(c);
 
 	if (c->message_string != NULL || c->prompt_string != NULL) {
 		/*
@@ -533,8 +608,10 @@ server_check_timers(struct client *c)
 void
 server_fill_clients(struct pollfd **pfd)
 {
-	struct client	*c;
-	u_int		 i;
+	struct client		*c;
+	struct window		*w;
+	struct window_pane	*wp;
+	u_int		 	 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
@@ -563,6 +640,20 @@ server_fill_clients(struct pollfd **pfd)
 		}
 		(*pfd)++;
 	}
+	
+	/*
+	 * Clear any window redraw flags (will have been redrawn as part of
+	 * client).
+	 */
+	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
+		w = ARRAY_ITEM(&windows, i);
+		if (w == NULL)
+			continue;
+		
+		w->flags &= ~WINDOW_REDRAW;
+		TAILQ_FOREACH(wp, &w->panes, entry)
+			wp->flags &= ~PANE_REDRAW;
+	}
 }
 
 /* Handle client pollfds. */
@@ -622,10 +713,10 @@ void
 server_handle_client(struct client *c)
 {
 	struct window_pane	*wp;
+	struct screen		*s;
 	struct timeval	 	 tv;
 	struct key_binding	*bd;
-	struct cmd		*cmd;
-	int		 	 key, prefix, status, xtimeout, can_repeat;
+	int		 	 key, prefix, status, xtimeout;
 	int			 mode;
 	u_char			 mouse[3];
 
@@ -646,7 +737,7 @@ server_handle_client(struct client *c)
 			return;
 		wp = c->session->curw->window->active;	/* could die */
 
-		server_clear_client_message(c);
+		status_message_clear(c);
 		if (c->prompt_string != NULL) {
 			status_prompt_key(c, key);
 			continue;
@@ -683,15 +774,8 @@ server_handle_client(struct client *c)
 			continue;
 		}
 
-		/* Check repeat flag. */
-		can_repeat = 1;
-		TAILQ_FOREACH(cmd, bd->cmdlist, qentry) {
-			if (!(cmd->entry->flags & CMD_CANREPEAT))
-				can_repeat = 0;
-		}
-
 		/* If already repeating, but this key can't repeat, skip it. */
-		if (c->flags & CLIENT_REPEAT && !can_repeat) {
+		if (c->flags & CLIENT_REPEAT && !bd->can_repeat) {
 			c->flags &= ~CLIENT_REPEAT;
 			if (key == prefix)
 				c->flags |= CLIENT_PREFIX;
@@ -701,7 +785,7 @@ server_handle_client(struct client *c)
 		}
 		
 		/* If this key can repeat, reset the repeat flags and timer. */
-		if (xtimeout != 0 && can_repeat) {
+		if (xtimeout != 0 && bd->can_repeat) {
 			c->flags |= CLIENT_PREFIX|CLIENT_REPEAT;
 
 			tv.tv_sec = xtimeout / 1000;
@@ -717,13 +801,14 @@ server_handle_client(struct client *c)
 	if (c->session == NULL)
 		return;
 	wp = c->session->curw->window->active;	/* could die - do each loop */
+	s = wp->screen;
 	
 	/* Ensure cursor position and mode settings. */
 	status = options_get_number(&c->session->options, "status");
-	if (wp->yoff + wp->screen->cy < c->sy - status)
-		tty_cursor(&c->tty, wp->screen->cx, wp->screen->cy, wp->yoff);
+	if (wp->yoff + s->cy < c->tty.sy - status)
+		tty_cursor(&c->tty, s->cx, s->cy, wp->xoff, wp->yoff);
 
-	mode = wp->screen->mode;
+	mode = s->mode;
 	if (server_locked)
 		mode &= ~TTY_NOCURSOR;
 	tty_update_mode(&c->tty, mode);
@@ -742,6 +827,8 @@ server_lost_client(struct client *c)
 
 	tty_free(&c->tty, c->flags & CLIENT_SUSPENDED);
 
+	screen_free(&c->status);
+
 	if (c->title != NULL)
 		xfree(c->title);
 
@@ -851,6 +938,7 @@ server_check_window(struct window *w)
 		else if (!flag) {
 			window_remove_pane(w, wp);
 			server_redraw_window(w);
+			layout_refresh(w, 0);
 		}
 		wp = wq;
 	} 
@@ -937,7 +1025,7 @@ server_second_timers(void)
 
 /* Update socket execute permissions based on whether sessions are attached. */
 int
-server_update_socket(const char *path)
+server_update_socket()
 {
 	struct session	*s;
 	u_int		 i;
@@ -956,9 +1044,9 @@ server_update_socket(const char *path)
 	if (n != last) {
 		last = n;
 		if (n != 0)
-			chmod(path, S_IRWXU);
+			chmod(socket_path, S_IRWXU);
 		else
-			chmod(path, S_IRUSR|S_IWUSR);
+			chmod(socket_path, S_IRUSR|S_IWUSR);
 	}
 
 	return (n);
