@@ -1,4 +1,4 @@
-/* $Id: status.c,v 1.71 2009/02/01 18:14:49 tcunha Exp $ */
+/* $Id: status.c,v 1.75 2009/02/13 21:39:45 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -38,14 +38,14 @@ void	status_prompt_add_history(struct client *);
 char   *status_prompt_complete(const char *);
 
 /* Draw status for client on the last lines of given context. */
-void
+int
 status_redraw(struct client *c)
 {
 	struct screen_write_ctx		ctx;
 	struct session		       *s = c->session;
 	struct winlink		       *wl;
 	struct window_pane	       *wp;
-	struct screen		       *sc = NULL;
+	struct screen		       *sc = NULL, old_status;
 	char		 	       *left, *right, *text, *ptr;
 	size_t				llen, rlen, offset, xx, yy, sy;
 	size_t				size, start, width;
@@ -54,14 +54,12 @@ status_redraw(struct client *c)
 
 	left = right = NULL;
 
-	/* Resize the target screen. */
-	if (screen_size_x(&c->status) != c->sx) {
-		screen_free(&c->status);
-		screen_init(&c->status, c->sx, 1, 0);
-	}
+	/* Create the target screen. */
+	memcpy(&old_status, &c->status, sizeof old_status);
+	screen_init(&c->status, c->tty.sx, 1, 0);
 
 	/* No status line? */
-	if (c->sy == 0 || !options_get_number(&s->options, "status"))
+	if (c->tty.sy == 0 || !options_get_number(&s->options, "status"))
 		goto off;
 	larrow = rarrow = 0;
 
@@ -72,7 +70,7 @@ status_redraw(struct client *c)
 	stdgc.fg = options_get_number(&s->options, "status-bg");
 	stdgc.attr |= options_get_number(&s->options, "status-attr");
 
-	yy = c->sy - 1;
+	yy = c->tty.sy - 1;
 	if (yy == 0)
 		goto blank;
 
@@ -100,9 +98,9 @@ status_redraw(struct client *c)
 		xx += llen + 1;
 	if (rlen != 0)
 		xx += rlen + 1;
-	if (c->sx == 0 || c->sx <= xx)
+	if (c->tty.sx == 0 || c->tty.sx <= xx)
 		goto blank;
-	xx = c->sx - xx;
+	xx = c->tty.sx - xx;
 
 	/*
 	 * Right. We have xx characters to fill. Find out how much is to go in
@@ -217,7 +215,7 @@ draw:
 
 	/* Draw the last item. */
 	if (rlen != 0) {
-		screen_write_cursormove(&ctx, c->sx - rlen - 1, yy);
+		screen_write_cursormove(&ctx, c->tty.sx - rlen - 1, yy);
 		screen_write_puts(&ctx, &stdgc, " %s", right);
 	}
 
@@ -237,9 +235,9 @@ draw:
 		if (rarrow == -1)
 			gc.attr ^= GRID_ATTR_REVERSE;
 		if (rlen != 0)
-			screen_write_cursormove(&ctx, c->sx - rlen - 2, yy);
+			screen_write_cursormove(&ctx, c->tty.sx - rlen - 2, yy);
 		else
-			screen_write_cursormove(&ctx, c->sx - 1, yy);
+			screen_write_cursormove(&ctx, c->tty.sx - 1, yy);
 		screen_write_putc(&ctx, &gc, '>');
 	}
 
@@ -249,7 +247,7 @@ blank:
  	/* Just draw the whole line as blank. */
 	screen_write_start(&ctx, NULL, &c->status);
 	screen_write_cursormove(&ctx, 0, yy);
-	for (offset = 0; offset < c->sx; offset++)
+	for (offset = 0; offset < c->tty.sx; offset++)
 		screen_write_putc(&ctx, &stdgc, ' ');
 
 	goto out;
@@ -269,13 +267,13 @@ off:
 	}
 
 	screen_write_cursormove(&ctx, 0, 0);
-	if (sy < c->sy) {
+	if (sy < c->tty.sy) {
 		/* If the screen is too small, use blank. */
- 		for (offset = 0; offset < c->sx; offset++)
+ 		for (offset = 0; offset < c->tty.sx; offset++)
  			screen_write_putc(&ctx, &stdgc, ' ');
 	} else {
 		screen_write_copy(&ctx,
-		    sc, 0, sc->grid->hsize + screen_size_y(sc) - 1, c->sx, 1);
+		    sc, 0, sc->grid->hsize + screen_size_y(sc) - 1, c->tty.sx, 1);
 	}
 
 out:
@@ -285,6 +283,13 @@ out:
 		xfree(left);
 	if (right != NULL)
 		xfree(right);
+
+	if (grid_compare(c->status.grid, old_status.grid) == 0) {
+		screen_free(&old_status);
+		return (0);
+	}
+	screen_free(&old_status);
+	return (1);
 }
 
 char *
@@ -483,25 +488,56 @@ status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
 	return (text);
 }
 
-/* Draw client message on status line of present else on last line. */
 void
+status_message_set(struct client *c, const char *msg)
+{
+	struct timeval	tv;
+	int		delay;
+
+	delay = options_get_number(&c->session->options, "display-time");
+	tv.tv_sec = delay / 1000;
+	tv.tv_usec = (delay % 1000) * 1000L;
+
+	c->message_string = xstrdup(msg);
+	if (gettimeofday(&c->message_timer, NULL) != 0)
+		fatal("gettimeofday");
+	timeradd(&c->message_timer, &tv, &c->message_timer);
+
+	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_STATUS;
+}
+
+void
+status_message_clear(struct client *c)
+{
+	if (c->message_string == NULL)
+		return;
+
+	xfree(c->message_string);
+	c->message_string = NULL;
+
+	c->tty.flags &= ~(TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_REDRAW;
+}
+
+/* Draw client message on status line of present else on last line. */
+int
 status_message_redraw(struct client *c)
 {
 	struct screen_write_ctx		ctx;
 	struct session		       *s = c->session;
+	struct screen		        old_status;
 	size_t			        len;
 	struct grid_cell		gc;
 
-	if (c->sx == 0 || c->sy == 0)
-		return;
-	if (screen_size_x(&c->status) != c->sx) {
-		screen_free(&c->status);
-		screen_init(&c->status, c->sx, 1, 0);
-	}
+	if (c->tty.sx == 0 || c->tty.sy == 0)
+		return (0);
+	memcpy(&old_status, &c->status, sizeof old_status);
+	screen_init(&c->status, c->tty.sx, 1, 0);
 
 	len = strlen(c->message_string);
-	if (len > c->sx)
-		len = c->sx;
+	if (len > c->tty.sx)
+		len = c->tty.sx;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
 	gc.bg = options_get_number(&s->options, "message-fg");
@@ -512,33 +548,81 @@ status_message_redraw(struct client *c)
 
 	screen_write_cursormove(&ctx, 0, 0);
 	screen_write_puts(&ctx, &gc, "%.*s", (int) len, c->message_string);
-	for (; len < c->sx; len++)
+	for (; len < c->tty.sx; len++)
 		screen_write_putc(&ctx, &gc, ' ');
 
 	screen_write_stop(&ctx);
+
+	if (grid_compare(c->status.grid, old_status.grid) == 0) {
+		screen_free(&old_status);
+		return (0);
+	}
+	screen_free(&old_status);
+	return (1);
+}
+
+void
+status_prompt_set(struct client *c,
+    const char *msg, int (*fn)(void *, const char *), void *data, int hide)
+{
+	c->prompt_string = xstrdup(msg);
+
+	c->prompt_buffer = xstrdup("");
+	c->prompt_index = 0;
+
+	c->prompt_callback = fn;
+	c->prompt_data = data;
+
+	c->prompt_hindex = 0;
+
+	c->prompt_hidden = hide;
+
+	mode_key_init(&c->prompt_mdata,
+	    options_get_number(&c->session->options, "status-keys"),
+	    MODEKEY_CANEDIT);
+
+	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_STATUS;
+}
+
+void
+status_prompt_clear(struct client *c)
+{
+	if (c->prompt_string == NULL)
+		return;
+
+	mode_key_free(&c->prompt_mdata);
+
+	xfree(c->prompt_string);
+	c->prompt_string = NULL;
+
+	xfree(c->prompt_buffer);
+	c->prompt_buffer = NULL;
+
+	c->tty.flags &= ~(TTY_NOCURSOR|TTY_FREEZE);
+	c->flags |= CLIENT_REDRAW;
 }
 
 /* Draw client prompt on status line of present else on last line. */
-void
+int
 status_prompt_redraw(struct client *c)
 {
 	struct screen_write_ctx	ctx;
 	struct session		       *s = c->session;
+	struct screen		        old_status;
 	size_t			        i, size, left, len, offset, n;
 	char				ch;
 	struct grid_cell		gc;
 
-	if (c->sx == 0 || c->sy == 0)
-		return;
-	if (screen_size_x(&c->status) != c->sx) {
-		screen_free(&c->status);
-		screen_init(&c->status, c->sx, 1, 0);
-	}
+	if (c->tty.sx == 0 || c->tty.sy == 0)
+		return (0);
+	memcpy(&old_status, &c->status, sizeof old_status);
+	screen_init(&c->status, c->tty.sx, 1, 0);
 	offset = 0;
 
 	len = strlen(c->prompt_string);
-	if (len > c->sx)
-		len = c->sx;
+	if (len > c->tty.sx)
+		len = c->tty.sx;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
 	gc.bg = options_get_number(&s->options, "message-fg");
@@ -550,7 +634,7 @@ status_prompt_redraw(struct client *c)
 	screen_write_cursormove(&ctx, 0, 0);
 	screen_write_puts(&ctx, &gc, "%.*s", (int) len, c->prompt_string);
 
-	left = c->sx - len;
+	left = c->tty.sx - len;
 	if (left != 0) {
 		if (c->prompt_index < left)
 			size = strlen(c->prompt_buffer);
@@ -571,7 +655,7 @@ status_prompt_redraw(struct client *c)
 			    "%.*s", (int) left, c->prompt_buffer + offset);
 		}
 
-		for (i = len + size; i < c->sx; i++)
+		for (i = len + size; i < c->tty.sx; i++)
 			screen_write_putc(&ctx, &gc, ' ');
 	}
 
@@ -587,6 +671,13 @@ status_prompt_redraw(struct client *c)
 	screen_write_putc(&ctx, &gc, ch);
 
 	screen_write_stop(&ctx);
+
+	if (grid_compare(c->status.grid, old_status.grid) == 0) {
+		screen_free(&old_status);
+		return (0);
+	}
+	screen_free(&old_status);
+	return (1);
 }
 
 /* Handle keys in prompt. */
@@ -597,35 +688,33 @@ status_prompt_key(struct client *c, int key)
 	size_t	size, n, off, idx;
 	char	word[64];
 
-	/* XXX This function needs some tidying... */
-
 	size = strlen(c->prompt_buffer);
-	switch (key) {
-	case KEYC_LEFT:
+	switch (mode_key_lookup(&c->prompt_mdata, key)) {
+	case MODEKEYCMD_LEFT:
 		if (c->prompt_index > 0) {
 			c->prompt_index--;
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case KEYC_RIGHT:
+	case MODEKEYCMD_RIGHT:
 		if (c->prompt_index < size) {
 			c->prompt_index++;
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case '\001':	/* C-a */
+	case MODEKEYCMD_STARTOFLINE:
 		if (c->prompt_index != 0) {
 			c->prompt_index = 0;
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case '\005':	/* C-e */
+	case MODEKEYCMD_ENDOFLINE:
 		if (c->prompt_index != size) {
 			c->prompt_index = size;
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case '\011':
+	case MODEKEYCMD_COMPLETE:
 		if (*c->prompt_buffer == '\0')
 			break;
 
@@ -673,8 +762,7 @@ status_prompt_key(struct client *c, int key)
 
 		c->flags |= CLIENT_STATUS;
 		break;
-	case '\010':
-	case '\177':
+	case MODEKEYCMD_BACKSPACE:
 		if (c->prompt_index != 0) {
 			if (c->prompt_index == size)
 				c->prompt_buffer[--c->prompt_index] = '\0';
@@ -687,7 +775,7 @@ status_prompt_key(struct client *c, int key)
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case KEYC_DC:
+ 	case MODEKEYCMD_DELETE:
 		if (c->prompt_index != size) {
 			memmove(c->prompt_buffer + c->prompt_index,
 			    c->prompt_buffer + c->prompt_index + 1,
@@ -695,7 +783,7 @@ status_prompt_key(struct client *c, int key)
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
-	case KEYC_UP:
+	case MODEKEYCMD_UP:
 		if (ARRAY_LENGTH(&c->prompt_hdata) == 0)
 			break;
 	       	xfree(c->prompt_buffer);
@@ -708,7 +796,7 @@ status_prompt_key(struct client *c, int key)
 		c->prompt_index = strlen(c->prompt_buffer);
 		c->flags |= CLIENT_STATUS;
 		break;
-	case KEYC_DOWN:
+	case MODEKEYCMD_DOWN:
 		xfree(c->prompt_buffer);
 
 		if (c->prompt_hindex != 0) {
@@ -722,21 +810,21 @@ status_prompt_key(struct client *c, int key)
 		c->prompt_index = strlen(c->prompt_buffer);
 		c->flags |= CLIENT_STATUS;
 		break;
- 	case '\r':	/* enter */
+ 	case MODEKEYCMD_CHOOSE:
 		if (*c->prompt_buffer != '\0') {
 			status_prompt_add_history(c);
 			if (c->prompt_callback(
 			    c->prompt_data, c->prompt_buffer) == 0)
-				server_clear_client_prompt(c);
+				status_prompt_clear(c);
 			break;
 		}
 		/* FALLTHROUGH */
-	case '\033':	/* escape */
+	case MODEKEYCMD_QUIT:
 		if (c->prompt_callback(c->prompt_data, NULL) == 0)
-			server_clear_client_prompt(c);
+			status_prompt_clear(c);
 		break;
-	default:
-		if (key < 32)
+	case MODEKEYCMD_OTHERKEY:
+		if (key < 32 || key > 126)
 			break;
 		c->prompt_buffer = xrealloc(c->prompt_buffer, 1, size + 2);
 
@@ -752,6 +840,8 @@ status_prompt_key(struct client *c, int key)
 
 		c->flags |= CLIENT_STATUS;
 		break;
+	default:
+		break;
 	}
 }
 
