@@ -1,4 +1,4 @@
-/* $Id: tmux.c,v 1.104 2009/01/30 00:24:49 nicm Exp $ */
+/* $Id: tmux.c,v 1.113 2009/04/20 19:25:58 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,7 +17,7 @@
  */
 
 #include <sys/types.h>
-#include <sys/wait.h>
+#include <sys/stat.h>
 
 #include <errno.h>
 #include <pwd.h>
@@ -46,6 +46,7 @@ const char	*_malloc_options = "AJX";
 volatile sig_atomic_t sigwinch;
 volatile sig_atomic_t sigterm;
 volatile sig_atomic_t sigcont;
+volatile sig_atomic_t sigchld;
 
 char		*cfg_file;
 struct options	 global_options;
@@ -58,9 +59,10 @@ time_t		 server_activity;
 int		 debug_level;
 int		 be_quiet;
 time_t		 start_time;
-const char	*socket_path;
+char		*socket_path;
 
 __dead void	 usage(void);
+char 		*makesockpath(const char *);
 
 #ifdef NO_PROGNAME
 const char      *__progname = "tmux";
@@ -69,8 +71,8 @@ const char      *__progname = "tmux";
 __dead void
 usage(void)
 {
-	fprintf(stderr, "usage: "
-	    "%s [-2dquVv] [-f file] [-S socket-path] [command [flags]]\n",
+	fprintf(stderr, "usage: %s [-28dqUuVv] [-f file] "
+	    "[-L socket-name] [-S socket-path] [command [flags]]\n",
 	    __progname);
 	exit(1);
 }
@@ -103,7 +105,7 @@ sighandler(int sig)
 		sigterm = 1;
 		break;
 	case SIGCHLD:
-		waitpid(WAIT_ANY, NULL, WNOHANG);
+		sigchld = 1;
 		break;
 	case SIGCONT:
 		sigcont = 1;
@@ -173,6 +175,34 @@ sigreset(void)
 		fatal("sigaction failed");
 }
 
+char *
+makesockpath(const char *label)
+{
+	char		base[MAXPATHLEN], *path;
+	struct stat	sb;
+	u_int		uid;
+
+	uid = getuid();
+	xsnprintf(base, MAXPATHLEN, "%s/%s-%d", _PATH_TMP, __progname, uid);
+
+	if (mkdir(base, S_IRWXU) != 0 && errno != EEXIST)
+		return (NULL);
+
+	if (lstat(base, &sb) != 0)
+		return (NULL);
+	if (!S_ISDIR(sb.st_mode)) {
+		errno = ENOTDIR;
+		return (NULL);
+	}
+	if (sb.st_uid != uid || (sb.st_mode & (S_IRWXG|S_IRWXO)) != 0) {
+		errno = EACCES;
+		return (NULL);
+	}
+
+	xasprintf(&path, "%s/%s", base, label);
+	return (path);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -185,21 +215,41 @@ main(int argc, char **argv)
 	struct hdr	 	 hdr;
 	const char		*shell;
 	struct passwd		*pw;
-	char			*path, *cause, *home, *pass = NULL;
-	char			 rpath[MAXPATHLEN], cwd[MAXPATHLEN];
+	char			*path, *label, *cause, *home, *pass = NULL;
+	char			 cwd[MAXPATHLEN];
 	int	 		 retcode, opt, flags, unlock, start_server;
 
 	unlock = flags = 0;
-	path = NULL;
-        while ((opt = getopt(argc, argv, "2df:qS:uUVv")) != -1) {
+	label = path = NULL;
+        while ((opt = getopt(argc, argv, "28df:L:qS:uUVv")) != -1) {
                 switch (opt) {
 		case '2':
 			flags |= IDENTIFY_256COLOURS;
+			flags &= ~IDENTIFY_88COLOURS;
+			break;
+		case '8':
+			flags |= IDENTIFY_88COLOURS;
+			flags &= ~IDENTIFY_256COLOURS;
 			break;
 		case 'f':
 			cfg_file = xstrdup(optarg);
 			break;
+		case 'L':
+			if (path != NULL) {
+				log_warnx("-L and -S cannot be used together");
+				exit(1);
+			}
+			if (label != NULL)
+				xfree(label);
+			label = xstrdup(optarg);
+			break;
 		case 'S':
+			if (label != NULL) {
+				log_warnx("-L and -S cannot be used together");
+				exit(1);
+			}
+			if (path != NULL)
+				xfree(path);
 			path = xstrdup(optarg);
 			break;
 		case 'q':
@@ -242,6 +292,7 @@ main(int argc, char **argv)
 	options_set_number(&global_options, "repeat-time", 500);
 	options_set_number(&global_options, "set-titles", 1);
 	options_set_number(&global_options, "lock-after-time", 0);
+	options_set_number(&global_options, "set-remain-on-exit", 0);
 	options_set_number(&global_options, "status", 1);
 	options_set_number(&global_options, "status-bg", 2);
 	options_set_number(&global_options, "status-fg", 0);
@@ -252,6 +303,7 @@ main(int argc, char **argv)
 	options_set_string(&global_options, "status-left", "[#S]");
 	options_set_string(
 	    &global_options, "status-right", "\"#24T\" %%H:%%M %%d-%%b-%%y");
+	options_set_number(&global_options, "status-keys", MODEKEY_EMACS);
 	options_init(&global_window_options, NULL);
 	options_set_number(&global_window_options, "aggressive-resize", 0);
 	options_set_number(&global_window_options, "clock-mode-colour", 4);
@@ -292,26 +344,13 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (path == NULL) {
-		xasprintf(&path,
-		    "%s/%s-%lu", _PATH_TMP, __progname, (u_long) getuid());
+	if (label == NULL)
+		label = xstrdup("default");
+	if (path == NULL && (path = makesockpath(label)) == NULL) {
+		log_warn("can't create socket");
+		exit(1);
 	}
-	if (realpath(path, rpath) == NULL) {
-		if (errno != ENOENT) {
-			log_warn("%s", path);
-			exit(1);
-		}
-		/*
-		 * Linux appears to fill in the buffer fine but then returns
-		 * ENOENT if the file doesn't exist. But since it returns an
-		 * error, we can't rely on the buffer. Grr.
-		 */
-		if (strlcpy(rpath, path, sizeof rpath) >= sizeof rpath) {
-			log_warnx("%s: %s", path, strerror(ENAMETOOLONG));
-			exit(1);
-		}
-	}
-	xfree(path);
+	xfree(label);
 
 	shell = getenv("SHELL");
 	if (shell == NULL || *shell == '\0') {
@@ -367,8 +406,9 @@ main(int argc, char **argv)
 	}
 	
  	memset(&cctx, 0, sizeof cctx);
-	if (client_init(rpath, &cctx, start_server, flags) != 0)
+	if (client_init(path, &cctx, start_server, flags) != 0)
 		exit(1);
+	xfree(path);
 
 	b = buffer_create(BUFSIZ);
 	if (unlock) {
@@ -398,7 +438,7 @@ main(int argc, char **argv)
 		}
 
 		if (buffer_poll(&pfd, cctx.srv_in, cctx.srv_out) != 0)
-			fatalx("lost server");
+			goto out;
 
 	restart:
 		if (BUFFER_USED(cctx.srv_in) < sizeof hdr)
