@@ -1,4 +1,4 @@
-/* $Id: tmux.h,v 1.259 2009/02/03 17:21:19 tcunha Exp $ */
+/* $Id: tmux.h,v 1.299 2009/04/03 17:21:46 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,7 +19,7 @@
 #ifndef TMUX_H
 #define TMUX_H
 
-#define PROTOCOL_VERSION -11
+#define PROTOCOL_VERSION -12
 
 /* Shut up gcc warnings about empty if bodies. */
 #define RB_AUGMENT(x) do {} while (0)
@@ -109,6 +109,7 @@ extern const char    *__progname;
 #define	_PATH_BSHELL	"/bin/sh"
 #define	_PATH_TMP	"/tmp/"
 #define _PATH_DEVNULL	"/dev/null"
+#define _PATH_TTY	"/dev/tty"
 #endif
 
 /* Default configuration file. */
@@ -117,14 +118,14 @@ extern const char    *__progname;
 /* Default prompt history length. */
 #define PROMPT_HISTORY 100
 
-/* Default environment. */
-#define CHILD_ENVIRON { NULL /* TMUX= */, "TERM=screen", NULL }
-
 /* Minimum pane size. */
 #define PANE_MINIMUM 4	/* includes separator line */
 
 /* Automatic name refresh interval, in milliseconds. */
-#define NAME_INTERVAL 100
+#define NAME_INTERVAL 500
+
+/* Escape timer period, in milliseconds. */
+#define ESCAPE_PERIOD 250
 
 /* Fatal errors. */
 #define fatal(msg) log_fatal("%s: %s", __func__, msg);
@@ -197,13 +198,21 @@ struct buffer {
 #define KEYC_F10 (KEYC_OFFSET + 0x10)
 #define KEYC_F11 (KEYC_OFFSET + 0x11)
 #define KEYC_F12 (KEYC_OFFSET + 0x12)
-#define KEYC_IC (KEYC_OFFSET + 0x13)
-#define KEYC_DC (KEYC_OFFSET + 0x14)
-#define KEYC_HOME (KEYC_OFFSET + 0x15)
-#define KEYC_END (KEYC_OFFSET + 0x16)
-#define KEYC_NPAGE (KEYC_OFFSET + 0x17)
-#define KEYC_PPAGE (KEYC_OFFSET + 0x18)
-#define KEYC_BTAB (KEYC_OFFSET + 0x19)
+#define KEYC_F13 (KEYC_OFFSET + 0x13)
+#define KEYC_F14 (KEYC_OFFSET + 0x14)
+#define KEYC_F15 (KEYC_OFFSET + 0x15)
+#define KEYC_F16 (KEYC_OFFSET + 0x16)
+#define KEYC_F17 (KEYC_OFFSET + 0x17)
+#define KEYC_F18 (KEYC_OFFSET + 0x18)
+#define KEYC_F19 (KEYC_OFFSET + 0x19)
+#define KEYC_F20 (KEYC_OFFSET + 0x1a)
+#define KEYC_IC (KEYC_OFFSET + 0x1b)
+#define KEYC_DC (KEYC_OFFSET + 0x1c)
+#define KEYC_HOME (KEYC_OFFSET + 0x1d)
+#define KEYC_END (KEYC_OFFSET + 0x1e)
+#define KEYC_NPAGE (KEYC_OFFSET + 0x1f)
+#define KEYC_PPAGE (KEYC_OFFSET + 0x20)
+#define KEYC_BTAB (KEYC_OFFSET + 0x21)
 
 /* Arrow keys. */
 #define KEYC_UP (KEYC_OFFSET + 0x50)
@@ -240,7 +249,6 @@ enum tty_code_code {
 	TTYC_CLEAR,	/* clear_screen, cl */
 	TTYC_CNORM,	/* cursor_normal, ve */
 	TTYC_COLORS,	/* max_colors, Co */
-	TTYC_CR,	/* carriage_return, cr */
 	TTYC_CSR,	/* change_scroll_region, cs */
 	TTYC_CUD,	/* parm_down_cursor, DO */
 	TTYC_CUD1,	/* cursor_down, do */
@@ -271,7 +279,15 @@ enum tty_code_code {
 	TTYC_KF1,	/* key_f1, k1 */
 	TTYC_KF10,	/* key_f10, k; */
 	TTYC_KF11,	/* key_f11, F1 */
-	TTYC_KF12,	/* key_f12, 21 */
+	TTYC_KF12,	/* key_f12, F2 */
+	TTYC_KF13,	/* key_f13, F3 */
+	TTYC_KF14,	/* key_f14, F4 */
+	TTYC_KF15,	/* key_f15, F5 */
+	TTYC_KF16,	/* key_f16, F6 */
+	TTYC_KF17,	/* key_f17, F7 */
+	TTYC_KF18,	/* key_f18, F8 */
+	TTYC_KF19,	/* key_f19, F9 */
+	TTYC_KF20,	/* key_f20, F10 */
 	TTYC_KF2,	/* key_f2, k2 */
 	TTYC_KF3,	/* key_f3, k3 */
 	TTYC_KF4,	/* key_f4, k4 */
@@ -331,7 +347,7 @@ struct tty_term_code_entry {
 
 /* Output commands. */
 enum tty_cmd {
-	TTY_CELL,
+	TTY_CELL,	
 	TTY_CLEARENDOFLINE,
 	TTY_CLEARENDOFSCREEN,
 	TTY_CLEARLINE,
@@ -343,6 +359,7 @@ enum tty_cmd {
 	TTY_INSERTCHARACTER,
 	TTY_INSERTLINE,
 	TTY_LINEFEED,
+	TTY_RAW,
 	TTY_REVERSEINDEX,
 };
 
@@ -385,7 +402,8 @@ struct msg_identify_data {
 
 #define IDENTIFY_UTF8 0x1
 #define IDENTIFY_256COLOURS 0x2
-#define IDENTIFY_HASDEFAULTS 0x4
+#define IDENTIFY_88COLOURS 0x4
+#define IDENTIFY_HASDEFAULTS 0x8
 	int		flags;
 
 	u_int		sx;
@@ -399,6 +417,42 @@ struct msg_resize_data {
 	u_int		sy;
 };
 
+/* Editing keys. */
+enum mode_key_cmd {
+	MODEKEYCMD_BACKSPACE,
+	MODEKEYCMD_CHOOSE,
+	MODEKEYCMD_CLEARSELECTION,
+	MODEKEYCMD_COMPLETE,
+	MODEKEYCMD_COPYSELECTION,
+	MODEKEYCMD_DELETE,
+	MODEKEYCMD_DOWN,
+	MODEKEYCMD_ENDOFLINE,
+	MODEKEYCMD_LEFT,
+	MODEKEYCMD_NEXTPAGE,
+	MODEKEYCMD_NEXTWORD,
+	MODEKEYCMD_NONE,
+	MODEKEYCMD_OTHERKEY,
+	MODEKEYCMD_PREVIOUSPAGE,
+	MODEKEYCMD_PREVIOUSWORD,
+	MODEKEYCMD_QUIT,
+	MODEKEYCMD_RIGHT,
+	MODEKEYCMD_STARTOFLINE,
+	MODEKEYCMD_STARTSELECTION,
+	MODEKEYCMD_UP,
+};
+
+struct mode_key_data {	
+	int			 type;
+
+	int			 flags;
+#define MODEKEY_EDITMODE 0x1
+#define MODEKEY_CANEDIT 0x2
+#define MODEKEY_CHOOSEMODE 0x4
+};
+
+#define MODEKEY_EMACS 0
+#define MODEKEY_VI 1
+
 /* Modes. */
 #define MODE_CURSOR 0x1
 #define MODE_INSERT 0x2
@@ -430,17 +484,24 @@ struct msg_resize_data {
 #define GRID_FLAG_PADDING 0x4
 #define GRID_FLAG_UTF8 0x8
 
-/* Grid cell. */
+/* Grid cell data. */
 struct grid_cell {
-	u_short	data;
 	u_char	attr;
 	u_char	flags;
 	u_char	fg;
 	u_char	bg;
+	u_char	data;
 } __packed;
 
-/* Grid data. */
-struct grid_data {
+/* Grid cell UTF-8 data. Used instead of data in grid_cell for UTF-8 cells. */
+#define UTF8_SIZE 8
+struct grid_utf8 {
+	u_char	width;
+	u_char	data[UTF8_SIZE];
+} __packed;
+
+/* Entire grid of cells. */
+struct grid {
 	u_int	sx;
 	u_int	sy;
 
@@ -449,6 +510,9 @@ struct grid_data {
 
 	u_int  *size;
 	struct grid_cell **data;
+
+	u_int  *usize;
+ 	struct grid_utf8 **udata;
 };
 
 /* Option data structures. */
@@ -491,14 +555,20 @@ struct screen_sel {
 struct screen {
 	char		*title;
 
-	struct grid_data *grid;		/* grid data */
+	struct grid     *grid;		/* grid data */
 
 	u_int		 cx;		/* cursor x */
 	u_int		 cy;		/* cursor y */
 
+	u_int		 old_cx;
+	u_int		 old_cy;
+
 	u_int		 rupper;	/* scroll region top */
 	u_int		 rlower;	/* scroll region bottom */
 
+	u_int		 old_rupper;
+	u_int		 old_rlower;
+
 	int		 mode;
 
 	struct screen_sel sel;
@@ -506,9 +576,7 @@ struct screen {
 
 /* Screen write context. */
 struct screen_write_ctx {
-	void		*data;
-	void		 (*write)(void *, enum tty_cmd, ...);
-
+	struct window_pane *wp;
 	struct screen	*s;
 };
 
@@ -533,7 +601,8 @@ struct input_ctx {
 	size_t		 len;
 	size_t		 off;
 
-	struct grid_cell cell;		/* current cell data */
+	struct grid_cell cell;
+	
 
 	struct grid_cell saved_cell;
 	u_int		 saved_cx;
@@ -581,14 +650,18 @@ struct window_pane {
 	u_int		 sx;
 	u_int		 sy;
 
+	u_int		 xoff;
 	u_int		 yoff;
 
 	int		 flags;
 #define PANE_HIDDEN 0x1
+#define PANE_RESTART 0x2
+#define PANE_REDRAW 0x4
 	
 	char		*cmd;
 	char		*cwd;
 
+	pid_t		 pid;
 	int		 fd;
 	char		 tty[TTY_NAME_MAX];
 	struct buffer	*in;
@@ -610,18 +683,19 @@ TAILQ_HEAD(window_panes, window_pane);
 struct window {
 	char		*name;
 	struct timeval	 name_timer;
-	pid_t		 pgrp;
 
 	struct window_pane *active;	
 	struct window_panes panes;
+	u_int		 layout;
 
 	u_int		 sx;
 	u_int		 sy;
-
+       
 	int		 flags;
 #define WINDOW_BELL 0x1
 #define WINDOW_HIDDEN 0x2
 #define WINDOW_ACTIVITY 0x4
+#define WINDOW_REDRAW 0x8
 
 	struct options	 options;
 
@@ -697,6 +771,8 @@ struct tty_term {
 
 #define TERM_HASDEFAULTS 0x1
 #define TERM_256COLOURS 0x2
+#define TERM_88COLOURS 0x4
+#define TERM_EARLYWRAP 0x8
 	int		 flags;
 
 	SLIST_ENTRY(tty_term) entry;
@@ -706,6 +782,9 @@ SLIST_HEAD(tty_terms, tty_term);
 struct tty {
 	char		*path;
 
+        u_int            sx;
+        u_int            sy;
+
 	u_int		 cx;
 	u_int		 cy;
 
@@ -756,9 +835,6 @@ struct client {
 	struct timeval	 status_timer;
 	struct timeval	 repeat_timer;
 
-	u_int		 sx;
-	u_int		 sy;
-
 	struct screen	 status;
 
 #define CLIENT_TERMINAL 0x1
@@ -781,6 +857,7 @@ struct client {
 	int		 prompt_hidden;
 	u_int		 prompt_hindex;
 	ARRAY_DECL(, char *) prompt_hdata;
+	struct mode_key_data prompt_mdata;
 
 	struct session	*session;
 };
@@ -828,12 +905,14 @@ struct cmd_entry {
 #define CMD_CANTNEST 0x2
 #define CMD_ARG1 0x4
 #define CMD_ARG01 0x8
-#define CMD_CANREPEAT 0x10
 #define CMD_KFLAG 0x10
 #define CMD_DFLAG 0x20
 #define CMD_GFLAG 0x40
 #define CMD_UFLAG 0x80
 #define CMD_AFLAG 0x100
+#define CMD_UPPERUFLAG 0x200
+#define CMD_UPPERDFLAG 0x400
+
 	int		 flags;
 
 	void		 (*init)(struct cmd *, int);
@@ -884,6 +963,7 @@ struct cmd_pane_data {
 struct key_binding {
 	int		 key;
 	struct cmd_list	*cmdlist;
+	int		 can_repeat;
 
 	SPLAY_ENTRY(key_binding) entry;
 };
@@ -909,31 +989,9 @@ struct set_option_entry {
 };
 extern const struct set_option_entry set_option_table[];
 extern const struct set_option_entry set_window_option_table[];
-#define NSETOPTION 22
+#define NSETOPTION 24
 #define NSETWINDOWOPTION 17
 
-/* Edit keys. */
-enum mode_key {
-	MODEKEY_BOL,
-	MODEKEY_CLEARSEL,
-	MODEKEY_COPYSEL,
-	MODEKEY_DOWN,
-	MODEKEY_ENTER,
-	MODEKEY_EOL,
-	MODEKEY_LEFT,
-	MODEKEY_NONE,
-	MODEKEY_NPAGE,
-	MODEKEY_NWORD,
-	MODEKEY_PPAGE,
-	MODEKEY_PWORD,
-	MODEKEY_QUIT,
-	MODEKEY_RIGHT,
-	MODEKEY_STARTSEL,
-	MODEKEY_UP,
-};
-#define MODEKEY_EMACS 0
-#define MODEKEY_VI 1
-
 #ifdef NO_STRTONUM
 /* strtonum.c */
 long long	 strtonum(const char *, long long, long long, const char **);
@@ -974,6 +1032,7 @@ char   *fgetln(FILE *, size_t *);
 extern volatile sig_atomic_t sigwinch;
 extern volatile sig_atomic_t sigterm;
 extern volatile sig_atomic_t sigcont;
+extern volatile sig_atomic_t sigchld;
 extern struct options global_options;
 extern struct options global_window_options;
 extern char	*cfg_file;
@@ -983,7 +1042,7 @@ extern time_t	 server_activity;
 extern int	 debug_level;
 extern int	 be_quiet;
 extern time_t	 start_time;
-extern const char *socket_path;
+extern char 	*socket_path;
 void		 logfile(const char *);
 void		 siginit(void);
 void		 sigreset(void);
@@ -993,7 +1052,9 @@ void		 sighandler(int);
 int		 load_cfg(const char *, char **x);
 
 /* mode-key.c */
-enum mode_key	 mode_key_lookup(int, int);
+void		 mode_key_init(struct mode_key_data *, int, int);
+void		 mode_key_free(struct mode_key_data *);
+enum mode_key_cmd mode_key_lookup(struct mode_key_data *, int);
 
 /* options.c */
 int	options_cmp(struct options_entry *, struct options_entry *);
@@ -1010,7 +1071,13 @@ void	options_set_number(struct options *, const char *, long long);
 long long options_get_number(struct options *, const char *);
 
 /* tty.c */
-void		 tty_cursor(struct tty *, u_int, u_int, u_int);
+void		 tty_emulate_repeat(struct tty *,
+    		     enum tty_code_code, enum tty_code_code, u_int);
+void		 tty_reset(struct tty *);
+void		 tty_region(struct tty *, u_int, u_int, u_int);
+void		 tty_cursor(struct tty *, u_int, u_int, u_int, u_int);
+void		 tty_cell(struct tty *,
+    		     const struct grid_cell *, const struct grid_utf8 *);
 void		 tty_putcode(struct tty *, enum tty_code_code);
 void		 tty_putcode1(struct tty *, enum tty_code_code, int);
 void		 tty_putcode2(struct tty *, enum tty_code_code, int, int);
@@ -1021,13 +1088,16 @@ void		 tty_start_tty(struct tty *);
 void		 tty_stop_tty(struct tty *);
 void		 tty_set_title(struct tty *, const char *);
 void		 tty_update_mode(struct tty *, int);
+void		 tty_draw_line(
+    		     struct tty *, struct screen *, u_int, u_int, u_int);
+void		 tty_redraw_region(struct tty *, struct window_pane *);
 int		 tty_open(struct tty *, char **);
 void		 tty_close(struct tty *, int);
 void		 tty_free(struct tty *, int);
-void		 tty_write(struct tty *,
-		     struct screen *, u_int, enum tty_cmd, ...);
-void		 tty_vwrite(struct tty *,
-		     struct screen *, u_int, enum tty_cmd, va_list);
+void		 tty_write(
+		     struct tty *, struct window_pane *, enum tty_cmd, ...);
+void		 tty_vwrite(
+    		     struct tty *, struct window_pane *, enum tty_cmd, va_list);
 
 /* tty-term.c */
 extern struct tty_terms tty_terms;
@@ -1050,9 +1120,8 @@ void		 tty_keys_free(struct tty *);
 int		 tty_keys_next(struct tty *, int *, u_char *);
 
 /* tty-write.c */
-void		 tty_write_window(void *, enum tty_cmd, ...);
-void		 tty_vwrite_window(void *, enum tty_cmd, va_list);
-void		 tty_write_update_mode(void *, int);
+void		 tty_write_cmd(struct window_pane *, enum tty_cmd, ...);
+void		 tty_write_mode(struct window_pane *, int);
 
 /* options-cmd.c */
 void	set_option_string(struct cmd_ctx *,
@@ -1106,6 +1175,7 @@ struct winlink	*cmd_find_window(
 extern const struct cmd_entry *cmd_table[];
 extern const struct cmd_entry cmd_attach_session_entry;
 extern const struct cmd_entry cmd_bind_key_entry;
+extern const struct cmd_entry cmd_break_pane_entry;
 extern const struct cmd_entry cmd_choose_session_entry;
 extern const struct cmd_entry cmd_choose_window_entry;
 extern const struct cmd_entry cmd_clock_mode_entry;
@@ -1134,6 +1204,7 @@ extern const struct cmd_entry cmd_lock_server_entry;
 extern const struct cmd_entry cmd_move_window_entry;
 extern const struct cmd_entry cmd_new_session_entry;
 extern const struct cmd_entry cmd_new_window_entry;
+extern const struct cmd_entry cmd_next_layout_entry;
 extern const struct cmd_entry cmd_next_window_entry;
 extern const struct cmd_entry cmd_paste_buffer_entry;
 extern const struct cmd_entry cmd_previous_window_entry;
@@ -1143,6 +1214,7 @@ extern const struct cmd_entry cmd_rename_window_entry;
 extern const struct cmd_entry cmd_resize_pane_down_entry;
 extern const struct cmd_entry cmd_resize_pane_up_entry;
 extern const struct cmd_entry cmd_respawn_window_entry;
+extern const struct cmd_entry cmd_rotate_window_entry;
 extern const struct cmd_entry cmd_save_buffer_entry;
 extern const struct cmd_entry cmd_scroll_mode_entry;
 extern const struct cmd_entry cmd_select_pane_entry;
@@ -1162,6 +1234,7 @@ extern const struct cmd_entry cmd_source_file_entry;
 extern const struct cmd_entry cmd_split_window_entry;
 extern const struct cmd_entry cmd_start_server_entry;
 extern const struct cmd_entry cmd_suspend_client_entry;
+extern const struct cmd_entry cmd_swap_pane_entry;
 extern const struct cmd_entry cmd_swap_window_entry;
 extern const struct cmd_entry cmd_switch_client_entry;
 extern const struct cmd_entry cmd_unbind_key_entry;
@@ -1228,7 +1301,7 @@ void	cmd_pane_free(struct cmd *);
 size_t	cmd_pane_print(struct cmd *, char *, size_t);
 
 /* client.c */
-int	 client_init(const char *, struct client_ctx *, int, int);
+int	 client_init(char *, struct client_ctx *, int, int);
 int	 client_flush(struct client_ctx *);
 int	 client_main(struct client_ctx *);
 
@@ -1246,7 +1319,7 @@ extern struct key_bindings key_bindings;
 int	 key_bindings_cmp(struct key_binding *, struct key_binding *);
 SPLAY_PROTOTYPE(key_bindings, key_binding, entry, key_bindings_cmp);
 struct key_binding *key_bindings_lookup(int);
-void	 key_bindings_add(int, struct cmd_list *);
+void	 key_bindings_add(int, int, struct cmd_list *);
 void	 key_bindings_remove(int);
 void	 key_bindings_init(void);
 void	 key_bindings_free(void);
@@ -1263,20 +1336,13 @@ const char *key_string_lookup_key(int);
 extern struct clients clients;
 struct client *server_create_client(int);
 int	 server_client_index(struct client *);
-int	 server_start(const char *);
+int	 server_start(char *);
 
 /* server-msg.c */
 int	 server_msg_dispatch(struct client *);
 
 /* server-fn.c */
-void	 server_set_client_message(struct client *, const char *);
-void	 server_clear_client_message(struct client *);
-void	 server_set_client_prompt(struct client *,
-	     const char *, int (*)(void *, const char *), void *, int);
-void	 server_clear_client_prompt(struct client *);
-struct session *server_extract_session(
-    	     struct msg_command_data *, char *, char **);
-void	 server_write(struct client *, enum hdrtype, const void *, size_t);
+const char **server_fill_environ(struct session *);
 void	 server_write_client(
              struct client *, enum hdrtype, const void *, size_t);
 void	 server_write_session(
@@ -1293,9 +1359,14 @@ void	 server_lock(void);
 int	 server_unlock(const char *);
 
 /* status.c */
-void	 status_redraw(struct client *);
-void	 status_message_redraw(struct client *);
-void	 status_prompt_redraw(struct client *);
+int	 status_redraw(struct client *);
+void	 status_message_set(struct client *, const char *);
+void	 status_message_clear(struct client *);
+int	 status_message_redraw(struct client *);
+void	 status_prompt_set(struct client *,
+	     const char *, int (*)(void *, const char *), void *, int);
+void	 status_prompt_clear(struct client *);
+int	 status_prompt_redraw(struct client *);
 void	 status_prompt_key(struct client *, int);
 
 /* resize.c */
@@ -1313,7 +1384,8 @@ void	 input_mouse(struct window_pane *, u_char, u_char, u_char);
 /* colour.c */
 const char *colour_tostring(u_char);
 int	 colour_fromstring(const char *);
-u_char	 colour_translate256(u_char);
+u_char	 colour_256to16(u_char);
+u_char	 colour_256to88(u_char);
 
 /* attributes.c */
 const char *attributes_tostring(u_char);
@@ -1321,43 +1393,45 @@ int	 attributes_fromstring(const char *);
 
 /* grid.c */
 extern const struct grid_cell grid_default_cell;
-struct grid_data *grid_create(u_int, u_int, u_int);
-void	 grid_destroy(struct grid_data *);
-void	 grid_reduce_line(struct grid_data *, u_int, u_int);
-void	 grid_expand_line(struct grid_data *, u_int, u_int);
-void	 grid_scroll_line(struct grid_data *);
-const struct grid_cell *grid_peek_cell(struct grid_data *, u_int, u_int);
-struct grid_cell *grid_get_cell(struct grid_data *, u_int, u_int);
-void	 grid_set_cell(
-	     struct grid_data *, u_int, u_int, const struct grid_cell *);
-void	 grid_clear(struct grid_data *, u_int, u_int, u_int, u_int);
-void	 grid_fill(struct grid_data *,
-    	     const struct grid_cell *, u_int, u_int, u_int, u_int);
-void	 grid_fill_lines(
-    	     struct grid_data *, const struct grid_cell *, u_int, u_int);
-void	 grid_clear_lines(struct grid_data *, u_int, u_int);
-void	 grid_move_lines(struct grid_data *, u_int, u_int, u_int);
-void	 grid_clear_cells(struct grid_data *, u_int, u_int, u_int);
-void	 grid_move_cells(struct grid_data *, u_int, u_int, u_int, u_int);
+struct grid *grid_create(u_int, u_int, u_int);
+void	 grid_destroy(struct grid *);
+int	 grid_compare(struct grid *, struct grid *);
+void	 grid_reduce_line(struct grid *, u_int, u_int);
+void	 grid_expand_line(struct grid *, u_int, u_int);
+void	 grid_expand_line_utf8(struct grid *, u_int, u_int);
+void	 grid_scroll_line(struct grid *);
+const struct grid_cell *grid_peek_cell(struct grid *, u_int, u_int);
+struct grid_cell *grid_get_cell(struct grid *, u_int, u_int);
+void	 grid_set_cell(struct grid *, u_int, u_int, const struct grid_cell *);
+const struct grid_utf8 *grid_peek_utf8(struct grid *, u_int, u_int);
+struct grid_utf8 *grid_get_utf8(struct grid *, u_int, u_int);
+void	 grid_set_utf8(struct grid *, u_int, u_int, const struct grid_utf8 *);
+void	 grid_clear(struct grid *, u_int, u_int, u_int, u_int);
+void	 grid_clear_lines(struct grid *, u_int, u_int);
+void	 grid_move_lines(struct grid *, u_int, u_int, u_int);
+void	 grid_clear_cells(struct grid *, u_int, u_int, u_int);
+void	 grid_move_cells(struct grid *, u_int, u_int, u_int, u_int);
 
 /* grid-view.c */
-const struct grid_cell *grid_view_peek_cell(struct grid_data *, u_int, u_int);
-struct grid_cell *grid_view_get_cell(struct grid_data *, u_int, u_int);
+const struct grid_cell *grid_view_peek_cell(struct grid *, u_int, u_int);
+struct grid_cell *grid_view_get_cell(struct grid *, u_int, u_int);
 void	 grid_view_set_cell(
-	     struct grid_data *, u_int, u_int, const struct grid_cell *);
-void	 grid_view_clear(struct grid_data *, u_int, u_int, u_int, u_int);
-void	 grid_view_fill(struct grid_data *,
-    	     const struct grid_cell *, u_int, u_int, u_int, u_int);
-void	 grid_view_scroll_region_up(struct grid_data *, u_int, u_int);
-void	 grid_view_scroll_region_down(struct grid_data *, u_int, u_int);
-void	 grid_view_insert_lines(struct grid_data *, u_int, u_int);
+    	     struct grid *, u_int, u_int, const struct grid_cell *);
+const struct grid_utf8 *grid_view_peek_utf8(struct grid *, u_int, u_int);
+struct grid_utf8 *grid_view_get_utf8(struct grid *, u_int, u_int);
+void	 grid_view_set_utf8(
+    	     struct grid *, u_int, u_int, const struct grid_utf8 *);
+void	 grid_view_clear(struct grid *, u_int, u_int, u_int, u_int);
+void	 grid_view_scroll_region_up(struct grid *, u_int, u_int);
+void	 grid_view_scroll_region_down(struct grid *, u_int, u_int);
+void	 grid_view_insert_lines(struct grid *, u_int, u_int);
 void	 grid_view_insert_lines_region(
-    	     struct grid_data *, u_int, u_int, u_int, u_int);
-void	 grid_view_delete_lines(struct grid_data *, u_int, u_int);
+    	     struct grid *, u_int, u_int, u_int, u_int);
+void	 grid_view_delete_lines(struct grid *, u_int, u_int);
 void	 grid_view_delete_lines_region(
-	     struct grid_data *, u_int, u_int, u_int, u_int);
-void	 grid_view_insert_cells(struct grid_data *, u_int, u_int, u_int);
-void	 grid_view_delete_cells(struct grid_data *, u_int, u_int, u_int);
+    	     struct grid *, u_int, u_int, u_int, u_int);
+void	 grid_view_insert_cells(struct grid *, u_int, u_int, u_int);
+void	 grid_view_delete_cells(struct grid *, u_int, u_int, u_int);
 
 /* screen-write.c */
 void	 screen_write_start(
@@ -1393,10 +1467,12 @@ void	 screen_write_kkeypadmode(struct screen_write_ctx *, int);
 void	 screen_write_clearendofscreen(struct screen_write_ctx *);
 void	 screen_write_clearstartofscreen(struct screen_write_ctx *);
 void	 screen_write_clearscreen(struct screen_write_ctx *);
-void	 screen_write_cell(struct screen_write_ctx *, const struct grid_cell *);
+void	 screen_write_cell(
+    	     struct screen_write_ctx *, const struct grid_cell *, u_char *);
 
 /* screen-redraw.c */
-void	 screen_redraw_screen(struct client *, struct screen *);
+void	 screen_redraw_screen(struct client *);
+void	 screen_redraw_pane(struct client *, struct window_pane *);
 void	 screen_redraw_status(struct client *);
 
 /* screen.c */
@@ -1429,6 +1505,7 @@ struct winlink	*winlink_previous(struct winlinks *, struct winlink *);
 void		 winlink_stack_push(struct winlink_stack *, struct winlink *);
 void		 winlink_stack_remove(struct winlink_stack *, struct winlink *);
 int	 	 window_index(struct window *, u_int *);
+struct window	*window_create1(u_int, u_int);
 struct window	*window_create(const char *, const char *,
 		     const char *, const char **, u_int, u_int, u_int, char **);
 void		 window_destroy(struct window *);
@@ -1457,6 +1534,10 @@ void		 window_pane_key(struct window_pane *, struct client *, int);
 void		 window_pane_mouse(struct window_pane *,
     		     struct client *, u_char, u_char, u_char);
 
+/* layout.c */
+void		 layout_refresh(struct window *, int);
+void		 layout_next(struct window *);
+
 /* window-clock.c */
 extern const struct window_mode window_clock_mode;
 
@@ -1517,7 +1598,7 @@ int	utf8_width(u_int);
 char   *section_string(char *, size_t, size_t, size_t);
 
 /* osdep-*.c */
-char   *get_argv0(int, char *);
+char   *osdep_get_name(int, char *);
 
 /* buffer.c */
 struct buffer 	*buffer_create(size_t);
