@@ -1,4 +1,4 @@
-/* $Id: tty-keys.c,v 1.21 2009/01/28 19:52:21 nicm Exp $ */
+/* $Id: tty-keys.c,v 1.25 2009/04/17 12:36:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -49,6 +49,14 @@ struct tty_key_ent tty_keys[] = {
 	{ TTYC_KF10,  NULL,     KEYC_F10,   TTYKEY_CTRL },
 	{ TTYC_KF11,  NULL,     KEYC_F11,   TTYKEY_CTRL },
 	{ TTYC_KF12,  NULL,     KEYC_F12,   TTYKEY_CTRL },
+	{ TTYC_KF13,  NULL,     KEYC_F13,   TTYKEY_CTRL },
+	{ TTYC_KF14,  NULL,     KEYC_F14,   TTYKEY_CTRL },
+	{ TTYC_KF15,  NULL,     KEYC_F15,   TTYKEY_CTRL },
+	{ TTYC_KF16,  NULL,     KEYC_F16,   TTYKEY_CTRL },
+	{ TTYC_KF17,  NULL,     KEYC_F17,   TTYKEY_CTRL },
+	{ TTYC_KF18,  NULL,     KEYC_F18,   TTYKEY_CTRL },
+	{ TTYC_KF19,  NULL,     KEYC_F19,   TTYKEY_CTRL },
+	{ TTYC_KF20,  NULL,     KEYC_F20,   TTYKEY_CTRL },
 	{ TTYC_KICH1, NULL,     KEYC_IC,    TTYKEY_CTRL },
 	{ TTYC_KDCH1, NULL,     KEYC_DC,    TTYKEY_CTRL },
 	{ TTYC_KHOME, NULL,     KEYC_HOME,  TTYKEY_CTRL },
@@ -123,7 +131,14 @@ tty_keys_cmp(struct tty_key *k1, struct tty_key *k2)
 void
 tty_keys_add(struct tty *tty, const char *s, int key, int flags)
 {
-	struct tty_key	*tk;
+	struct tty_key	*tk, tl;
+
+	tl.string = s;
+	if ((tk = RB_FIND(tty_keys, &tty->ktree, &tl)) != NULL) {
+		log_debug("already key matching: %s (old %x, new %x)", 
+		    tk->string, tk->key, key);
+		return;
+	}
 
 	tk = xmalloc(sizeof *tk);
 	tk->string = xstrdup(s);
@@ -174,10 +189,10 @@ tty_keys_init(struct tty *tty)
 void
 tty_keys_free(struct tty *tty)
 {
-	struct tty_key	*tk, *tl;
-
-	for (tk = RB_MIN(tty_keys, &tty->ktree); tk != NULL; tk = tl) {
-		tl = RB_NEXT(tty_keys, &tty->ktree, tk);
+	struct tty_key	*tk;
+	
+	while (!RB_EMPTY(&tty->ktree)) {
+		tk = RB_ROOT(&tty->ktree);
 		RB_REMOVE(tty_keys, &tty->ktree, tk);
 		xfree(tk->string);
 		xfree(tk);
@@ -232,41 +247,35 @@ tty_keys_next(struct tty *tty, int *key, u_char *mouse)
 	/* If a normal key, return it. */
 	if (*buf != '\033') {
 		*key = buffer_read8(tty->in);
-		return (0);
+		goto found;
 	}
 
 	/* Look for matching key string and return if found. */
 	tk = tty_keys_find(tty, buf + 1, len - 1, &size);
 	if (tk != NULL) {
-		*key = tk->key;
 		buffer_remove(tty->in, size + 1);
-
-		tty->flags &= ~TTY_ESCAPE;
-		return (0);
+		*key = tk->key;
+		goto found;
 	}
 
 	/* Not found. Is this a mouse key press? */
 	*key = tty_keys_parse_mouse(tty, buf, len, &size, mouse);
 	if (*key != KEYC_NONE) {
 		buffer_remove(tty->in, size);
-
-		tty->flags &= ~TTY_ESCAPE;
-		return (0);
+		goto found;
 	}
 
 	/* Not found. Try to parse xterm-type arguments. */
 	*key = tty_keys_parse_xterm(tty, buf, len, &size);
 	if (*key != KEYC_NONE) {
 		buffer_remove(tty->in, size);
-
-		tty->flags &= ~TTY_ESCAPE;
-		return (0);
+		goto found;
 	}
 	
 	/* Escape but no key string. If the timer isn't started, start it. */
 	if (!(tty->flags & TTY_ESCAPE)) {
 		tv.tv_sec = 0;
-		tv.tv_usec = 500 * 1000L;
+		tv.tv_usec = ESCAPE_PERIOD * 1000L;
 		if (gettimeofday(&tty->key_timer, NULL) != 0)
 			fatal("gettimeofday");
 		timeradd(&tty->key_timer, &tv, &tty->key_timer);
@@ -275,41 +284,39 @@ tty_keys_next(struct tty *tty, int *key, u_char *mouse)
 		return (1);
 	}
 
-	/* Otherwise, if the timer hasn't expired, wait. */
+	/* Skip the escape. */
+	buf++;
+	len--;
+
+	/* Is there a normal key following? */
+	if (len != 0 && *buf != '\033') {
+		buffer_remove(tty->in, 1);
+		*key = KEYC_ADDESC(buffer_read8(tty->in));
+		goto found;
+	} 
+
+	/* Or a key string? */
+	if (len > 1) {
+		tk = tty_keys_find(tty, buf + 1, len - 1, &size);
+		if (tk != NULL) {
+			buffer_remove(tty->in, size + 2);
+			*key = KEYC_ADDESC(tk->key);
+			goto found;
+		}
+	}
+
+	/* If the timer hasn't expired, keep waiting. */
 	if (gettimeofday(&tv, NULL) != 0)
 		fatal("gettimeofday");
-	if (!timercmp(&tty->key_timer, &tv, >))
+	if (timercmp(&tty->key_timer, &tv, >))
 		return (1);
+	
+	/* Give up and return the escape. */
+	buffer_remove(tty->in, 1);
+	*key = '\033';
+
+found:
 	tty->flags &= ~TTY_ESCAPE;
-
-	/* Remove the leading escape. */
-	buffer_remove(tty->in, 1);
-	buf = BUFFER_OUT(tty->in);
-	len = BUFFER_USED(tty->in);
-
-	/* If we have no following data, return escape. */
-	if (len == 0) {
-		*key = '\033';
-		return (0);
-	}
-
-	/* If a normal key follows, return it. */
-	if (*buf != '\033') {
-		*key = KEYC_ADDESC(buffer_read8(tty->in));
-		return (0);
-	}
-
-	/* Try to look up the key. */
-	tk = tty_keys_find(tty, buf + 1, len - 1, &size);
-	if (tk != NULL) {
-		*key = KEYC_ADDESC(tk->key);
- 		buffer_remove(tty->in, size + 1);
-		return (0);
-	}
-
-	/* If not found, return escape-escape. */
-	*key = KEYC_ADDESC('\033');
-	buffer_remove(tty->in, 1);
 	return (0);
 }
 
@@ -398,7 +405,7 @@ tty_keys_parse_xterm(struct tty *tty, char *buf, size_t len, size_t *size)
 		key = KEYC_ADDESC(key);
 		break;
 	case '2':
-	key = KEYC_ADDSFT(key);
+		key = KEYC_ADDSFT(key);
 		break;
 	}
 
