@@ -1,4 +1,4 @@
-/* $Id: tty-term.c,v 1.9 2009/02/01 18:11:51 nicm Exp $ */
+/* $Id: tty-term.c,v 1.17 2009/03/07 10:29:06 nicm Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -39,7 +39,6 @@ struct tty_term_code_entry tty_term_codes[NTTYCODE] = {
 	{ TTYC_CLEAR, TTYCODE_STRING, "clear" },
 	{ TTYC_CNORM, TTYCODE_STRING, "cnorm" },
 	{ TTYC_COLORS, TTYCODE_NUMBER, "colors" },
-	{ TTYC_CR, TTYCODE_STRING, "cr" },
 	{ TTYC_CSR, TTYCODE_STRING, "csr" },
 	{ TTYC_CUD, TTYCODE_STRING, "cud" },
 	{ TTYC_CUD1, TTYCODE_STRING, "cud1" },
@@ -71,6 +70,14 @@ struct tty_term_code_entry tty_term_codes[NTTYCODE] = {
 	{ TTYC_KF10, TTYCODE_STRING, "kf10" },
 	{ TTYC_KF11, TTYCODE_STRING, "kf11" },
 	{ TTYC_KF12, TTYCODE_STRING, "kf12" },
+	{ TTYC_KF13, TTYCODE_STRING, "kf13" },
+	{ TTYC_KF14, TTYCODE_STRING, "kf14" },
+	{ TTYC_KF15, TTYCODE_STRING, "kf15" },
+	{ TTYC_KF16, TTYCODE_STRING, "kf16" },
+	{ TTYC_KF17, TTYCODE_STRING, "kf17" },
+	{ TTYC_KF18, TTYCODE_STRING, "kf18" },
+	{ TTYC_KF19, TTYCODE_STRING, "kf19" },
+	{ TTYC_KF20, TTYCODE_STRING, "kf20" },
 	{ TTYC_KF2, TTYCODE_STRING, "kf2" },
 	{ TTYC_KF3, TTYCODE_STRING, "kf3" },
 	{ TTYC_KF4, TTYCODE_STRING, "kf4" },
@@ -149,6 +156,26 @@ tty_term_quirks(struct tty_term *term)
 			term->codes[TTYC_ICH1].value.string = xstrdup("\033[@");
 		}
 	}
+
+#ifdef __FreeBSD__
+	if (strncmp(term->name, "cons", 4) == 0) {
+		/*
+		 * FreeBSD's console wraps lines at $COLUMNS - 1 rather than
+		 * $COLUMNS (the cursor can never be beyond $COLUMNS - 1) and
+		 * does not appear to support changing this behaviour, or any
+		 * of the obvious possibilities (turning off right margin
+		 * wrapping, insert mode).
+		 *
+		 * This is irritating, most notably because it is impossible to
+		 * write to the very bottom-right of the screen without
+		 * scrolling.
+		 *
+		 * Flag the terminal here and apply some workarounds in other
+		 * places to do the best possible.
+		 */
+		term->flags |= TERM_EARLYWRAP;
+	}
+#endif
 }
 
 struct tty_term *
@@ -210,8 +237,6 @@ tty_term_find(char *name, int fd, char **cause)
 				break;
 			code->type = TTYCODE_STRING;
 			code->value.string = tty_term_strip(s);
-			log_debug(
-			    "code %d,%s: (string) %s", ent->code, ent->name, s);
 			break;
 		case TTYCODE_NUMBER:
 			n = tigetnum(ent->name);
@@ -219,8 +244,6 @@ tty_term_find(char *name, int fd, char **cause)
 				break;
 			code->type = TTYCODE_NUMBER;
 			code->value.number = n;
-			log_debug(
-			    "code %d,%s: (number) %d", ent->code, ent->name, n);
 			break;
 		case TTYCODE_FLAG:
 			n = tigetflag(ent->name);
@@ -228,8 +251,6 @@ tty_term_find(char *name, int fd, char **cause)
 				break;
 			code->type = TTYCODE_FLAG;
 			code->value.number = n;
-			log_debug(
-			    "code %d,%s: (flag) %d", ent->code, ent->name, n);
 			break;
 		}
 	}
@@ -243,18 +264,10 @@ tty_term_find(char *name, int fd, char **cause)
 		xasprintf(cause, "terminal does not support clear");
 		goto error;
 	}
-	if (!tty_term_has(term, TTYC_CR)) {
-		xasprintf(cause, "terminal does not support cr");
-		goto error;
-	}
 	if (!tty_term_has(term, TTYC_RI)) {
 		xasprintf(cause, "terminal does not support ri");
 		goto error;
 	}
-	if (!tty_term_has(term, TTYC_CSR)) {
-		xasprintf(cause, "terminal does not support csr");
-		goto error;
-	}
 	if (!tty_term_has(term, TTYC_CUP)) {
 		xasprintf(cause, "terminal does not support cup");
 		goto error;
@@ -297,7 +310,7 @@ tty_term_find(char *name, int fd, char **cause)
 		term->flags |= TERM_HASDEFAULTS;
 
 	/*
-	 * Try to figure out if we have 256 colours. The standard xterm
+	 * Try to figure out if we have 256 or 88 colours. The standard xterm
 	 * definitions are broken (well, or the way they are parsed is: in any
 	 * case they end up returning 8). So also do a hack.
 	 */
@@ -305,6 +318,10 @@ tty_term_find(char *name, int fd, char **cause)
 		term->flags |= TERM_256COLOURS;
 	if (strstr(name, "256col") != NULL) /* XXX HACK */
 		term->flags |= TERM_256COLOURS;
+	if (tty_term_number(term, TTYC_COLORS) == 88)
+		term->flags |= TERM_88COLOURS;
+	if (strstr(name, "88col") != NULL) /* XXX HACK */
+		term->flags |= TERM_88COLOURS;
 
 	return (term);
 
