@@ -1,4 +1,4 @@
-/* $Id: tty.c,v 1.65 2009/02/07 00:05:23 nicm Exp $ */
+/* $Id: tty.c,v 1.92 2009/04/02 21:08:15 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,36 +30,31 @@
 void	tty_fill_acs(struct tty *);
 u_char	tty_get_acs(struct tty *, u_char);
 
-void	tty_emulate_repeat(
-    	    struct tty *, enum tty_code_code, enum tty_code_code, u_int);
-
 void	tty_raw(struct tty *, const char *);
 
-void	tty_reset(struct tty *);
-void	tty_region(struct tty *, struct screen *, u_int);
+int	tty_try_256(struct tty *, u_char, const char *);
+int	tty_try_88(struct tty *, u_char, const char *);
+
 void	tty_attributes(struct tty *, const struct grid_cell *);
 void	tty_attributes_fg(struct tty *, const struct grid_cell *);
 void	tty_attributes_bg(struct tty *, const struct grid_cell *);
 
-void	tty_cmd_bell(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_carriagereturn(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_cell(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearendofline(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearendofscreen(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearline(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearscreen(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearstartofline(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_clearstartofscreen(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_deletecharacter(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_deleteline(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_insertcharacter(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_insertline(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_insertmode(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_linefeed(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_mousemode(struct tty *, struct screen *, u_int, va_list);
-void	tty_cmd_reverseindex(struct tty *, struct screen *, u_int, va_list);
+void	tty_cmd_cell(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearendofline(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearendofscreen(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearline(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearscreen(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearstartofline(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_clearstartofscreen(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_deletecharacter(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_deleteline(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_insertcharacter(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_insertline(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_linefeed(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_raw(struct tty *, struct window_pane *, va_list);
+void	tty_cmd_reverseindex(struct tty *, struct window_pane *, va_list);
 
-void (*tty_cmds[])(struct tty *, struct screen *, u_int, va_list) = {
+void (*tty_cmds[])(struct tty *, struct window_pane *, va_list) = {
 	tty_cmd_cell,
 	tty_cmd_clearendofline,
 	tty_cmd_clearendofscreen,
@@ -72,6 +67,7 @@ void (*tty_cmds[])(struct tty *, struct screen *, u_int, va_list) = {
 	tty_cmd_insertcharacter,
 	tty_cmd_insertline,
 	tty_cmd_linefeed,
+	tty_cmd_raw,
 	tty_cmd_reverseindex,
 };
 
@@ -99,9 +95,11 @@ tty_open(struct tty *tty, char **cause)
 	}
 
 	if ((mode = fcntl(tty->fd, F_GETFL)) == -1)
-		fatal("fcntl");
+		fatal("fcntl failed");
 	if (fcntl(tty->fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl");
+		fatal("fcntl failedo");
+	if (fcntl(tty->fd, F_SETFD, FD_CLOEXEC) == -1)
+		fatal("fcntl failed");
 
 	if (debug_level > 3)
 		tty->log_fd = open("tmux.out", O_WRONLY|O_CREAT|O_TRUNC, 0644);
@@ -192,25 +190,21 @@ tty_stop_tty(struct tty *tty)
 	 * because the fd is invalid. Things like ssh -t can easily leave us
 	 * with a dead tty.
 	 */
-	if (ioctl(tty->fd, TIOCGWINSZ, &ws) == -1) {
-		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
-			fatal("ioctl(TIOCGWINSZ)");
-	} else if (tcsetattr(tty->fd, TCSANOW, &tty->tio) == -1) {
-		if (errno != EBADF && errno != ENXIO && errno != ENOTTY)
-			fatal("tcsetattr failed");
-	} else {
-		tty_raw(tty,
-		    tty_term_string2(tty->term, TTYC_CSR, 0, ws.ws_row - 1));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMACS));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_SGR0));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_CLEAR));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMKX));
-		tty_raw(tty, tty_term_string(tty->term, TTYC_RMCUP));
+	if (ioctl(tty->fd, TIOCGWINSZ, &ws) == -1)
+		return;
+	if (tcsetattr(tty->fd, TCSANOW, &tty->tio) == -1)
+		return;
 
-		tty_raw(tty, tty_term_string(tty->term, TTYC_CNORM));
-		if (tty_term_has(tty->term, TTYC_KMOUS))
-			tty_raw(tty, "\033[?1000l");
-	}
+	tty_raw(tty, tty_term_string2(tty->term, TTYC_CSR, 0, ws.ws_row - 1));
+	tty_raw(tty, tty_term_string(tty->term, TTYC_RMACS));
+	tty_raw(tty, tty_term_string(tty->term, TTYC_SGR0));
+	tty_raw(tty, tty_term_string(tty->term, TTYC_CLEAR));
+	tty_raw(tty, tty_term_string(tty->term, TTYC_RMKX));
+	tty_raw(tty, tty_term_string(tty->term, TTYC_RMCUP));
+	
+	tty_raw(tty, tty_term_string(tty->term, TTYC_CNORM));
+	if (tty_term_has(tty->term, TTYC_KMOUS))
+		tty_raw(tty, "\033[?1000l");
 }
 
 void
@@ -250,7 +244,7 @@ tty_close(struct tty *tty, int no_stop)
 
 	if (!no_stop)
 		tty_stop_tty(tty);
-
+	
 	tty_term_free(tty->term);
 	tty_keys_free(tty);
 
@@ -314,12 +308,23 @@ tty_puts(struct tty *tty, const char *s)
 void
 tty_putc(struct tty *tty, char ch)
 {
+	u_int	sx;
+
 	if (tty->cell.attr & GRID_ATTR_CHARSET)
 		ch = tty_get_acs(tty, ch);
 	buffer_write8(tty->out, ch);
 
-	if (ch >= 0x20)
-		tty->cx++;	/* This is right most of the time. */
+	if (ch >= 0x20) {
+		sx = tty->sx;
+		if (tty->term->flags & TERM_EARLYWRAP)
+			sx--;
+
+		if (tty->cx == sx) {
+			tty->cx = 0;
+			tty->cy++;
+		} else
+			tty->cx++;
+	}
 
 	if (tty->log_fd != -1)
 		write(tty->log_fd, &ch, 1);
@@ -374,269 +379,419 @@ tty_emulate_repeat(
 	}
 }
 
+/*
+ * Redraw scroll region using data from screen (already updated). Used when
+ * CSR not supported, or window is a pane that doesn't take up the full
+ * width of the terminal.
+ */
 void
-tty_write(struct tty *tty, struct screen *s, u_int oy, enum tty_cmd cmd, ...)
+tty_redraw_region(struct tty *tty, struct window_pane *wp)
+{
+	struct screen	*s = wp->screen;
+	u_int		 i;
+
+	/*
+	 * If region is >= 50% of the screen, just schedule a window redraw. In
+	 * most cases, this is likely to be followed by some more scrolling -
+	 * without this, the entire pane ends up being redrawn many times which
+	 * can be much more data.
+	 */
+	if (s->old_rupper - s->old_rlower >= screen_size_y(s) / 2) {
+		wp->flags |= PANE_REDRAW;
+		return;
+	}
+
+	if (s->old_cy < s->old_rupper || s->old_cy > s->old_rlower) {
+		for (i = s->old_cy; i < screen_size_y(s); i++)
+			tty_draw_line(tty, s, i, wp->xoff, wp->yoff);
+	} else {
+		for (i = s->old_rupper; i <= s->old_rlower; i++)
+			tty_draw_line(tty, s, i, wp->xoff, wp->yoff);
+	}
+}
+
+void
+tty_draw_line(struct tty *tty, struct screen *s, u_int py, u_int ox, u_int oy)
+{
+	const struct grid_cell	*gc;
+	const struct grid_utf8	*gu;
+	u_int			 i, sx;
+
+	sx = screen_size_x(s);
+	if (sx > s->grid->size[s->grid->hsize + py])
+		sx = s->grid->size[s->grid->hsize + py];
+	if (sx > tty->sx)
+		sx = tty->sx;
+
+	tty_cursor(tty, 0, py, ox, oy);
+	for (i = 0; i < sx; i++) {
+		gc = grid_view_peek_cell(s->grid, i, py);
+
+		gu = NULL;
+		if (gc->flags & GRID_FLAG_UTF8)
+			gu = grid_view_peek_utf8(s->grid, i, py);
+
+		if (screen_check_selection(s, i, py)) {
+			s->sel.cell.data = gc->data;
+			tty_cell(tty, &s->sel.cell, gu);
+		} else
+			tty_cell(tty, gc, gu);
+	}
+
+	if (sx >= tty->sx)
+		return;
+	tty_reset(tty);
+		
+	tty_cursor(tty, sx, py, ox, oy);
+	if (screen_size_x(s) >= tty->sx && tty_term_has(tty->term, TTYC_EL))
+		tty_putcode(tty, TTYC_EL);
+	else {
+		for (i = sx; i < screen_size_x(s); i++)
+			tty_putc(tty, ' ');
+	}
+}
+
+void
+tty_write(struct tty *tty, struct window_pane *wp, enum tty_cmd cmd, ...)
 {
 	va_list	ap;
 
 	va_start(ap, cmd);
-	tty_vwrite(tty, s, oy, cmd, ap);
+	tty_vwrite(tty, wp, cmd, ap);
 	va_end(ap);
 }
 
 void
 tty_vwrite(
-    struct tty *tty, struct screen *s, u_int oy, enum tty_cmd cmd, va_list ap)
+    struct tty *tty, struct window_pane *wp, enum tty_cmd cmd, va_list ap)
 {
 	if (tty->flags & TTY_FREEZE || tty->term == NULL)
 		return;
 	if (tty_cmds[cmd] != NULL)
-		tty_cmds[cmd](tty, s, oy, ap);
+		tty_cmds[cmd](tty, wp, ap);
 }
 
 void
-tty_cmd_insertcharacter(
-    struct tty *tty, unused struct screen *s, u_int oy, va_list ap)
+tty_cmd_insertcharacter(struct tty *tty, struct window_pane *wp, va_list ap)
 {
-	u_int	ua;
+	struct screen	*s = wp->screen;
+	u_int		 ua;
+
+	if (wp->xoff != 0 || screen_size_x(s) < tty->sx) {
+		tty_draw_line(tty, wp->screen, s->old_cy, wp->xoff, wp->yoff);
+		return;
+	}
 
 	ua = va_arg(ap, u_int);
 
 	tty_reset(tty);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
+ 	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 	if (tty_term_has(tty->term, TTYC_ICH) || 
 	    tty_term_has(tty->term, TTYC_ICH1))
-	    tty_emulate_repeat(tty, TTYC_ICH, TTYC_ICH1, ua);
+		tty_emulate_repeat(tty, TTYC_ICH, TTYC_ICH1, ua);
 	else {
 		tty_putcode(tty, TTYC_SMIR);
 		while (ua-- > 0)
 			tty_putc(tty, ' ');
 		tty_putcode(tty, TTYC_RMIR);
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 	}
 }
 
 void
-tty_cmd_deletecharacter(
-    struct tty *tty, unused struct screen *s, u_int oy, va_list ap)
+tty_cmd_deletecharacter(struct tty *tty, struct window_pane *wp, va_list ap)
 {
-	u_int	ua;
+	struct screen	*s = wp->screen;
+	u_int		 ua;
+
+	if (wp->xoff != 0 || screen_size_x(s) < tty->sx) {
+		tty_draw_line(tty, wp->screen, s->old_cy, wp->xoff, wp->yoff);
+		return;
+	}
 
 	ua = va_arg(ap, u_int);
 
 	tty_reset(tty);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
+ 	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 	tty_emulate_repeat(tty, TTYC_DCH, TTYC_DCH1, ua);
 }
 
 void
-tty_cmd_insertline(
-    struct tty *tty, unused struct screen *s, u_int oy, va_list ap)
+tty_cmd_insertline(struct tty *tty, struct window_pane *wp, va_list ap)
 {
-	u_int	ua;
+	struct screen	*s = wp->screen;
+	u_int		 ua;
+
+ 	if (wp->xoff != 0 || screen_size_x(s) < tty->sx ||
+	    !tty_term_has(tty->term, TTYC_CSR)) {
+		tty_redraw_region(tty, wp);
+		return;
+	}
 
 	ua = va_arg(ap, u_int);
 
 	tty_reset(tty);
- 	tty_region(tty, s, oy);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
+ 	tty_region(tty, s->old_rupper, s->old_rlower, wp->yoff);
+
+ 	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 	tty_emulate_repeat(tty, TTYC_IL, TTYC_IL1, ua);
 }
 
 void
-tty_cmd_deleteline(
-    struct tty *tty, unused struct screen *s, u_int oy, va_list ap)
+tty_cmd_deleteline(struct tty *tty, struct window_pane *wp, va_list ap)
 {
-	u_int	ua;
+	struct screen	*s = wp->screen;
+	u_int		 ua;
+
+ 	if (wp->xoff != 0 || screen_size_x(s) < tty->sx ||
+	    !tty_term_has(tty->term, TTYC_CSR)) {
+		tty_redraw_region(tty, wp);
+		return;
+	}
 
 	ua = va_arg(ap, u_int);
 
 	tty_reset(tty);
- 	tty_region(tty, s, oy);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
+ 	tty_region(tty, s->old_rupper, s->old_rlower, wp->yoff);
+
+ 	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 	tty_emulate_repeat(tty, TTYC_DL, TTYC_DL1, ua);
 }
 
 void
-tty_cmd_clearline(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+tty_cmd_clearline(struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i;
+	struct screen	*s = wp->screen;
+	u_int		 i;
 
 	tty_reset(tty);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
-	if (tty_term_has(tty->term, TTYC_EL)) {
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, 0);
+ 	tty_cursor(tty, 0, s->old_cy, wp->xoff, wp->yoff);
+	if (wp->xoff == 0 && screen_size_x(s) >= tty->sx &&
+	    tty_term_has(tty->term, TTYC_EL)) {
 		tty_putcode(tty, TTYC_EL);
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 	} else {
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, 0);
 		for (i = 0; i < screen_size_x(s); i++)
 			tty_putc(tty, ' ');
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 	}
 }
 
 void
 tty_cmd_clearendofline(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+    struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i;
+	struct screen	*s = wp->screen;
+	u_int		 i;
 
 	tty_reset(tty);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
-	if (tty_term_has(tty->term, TTYC_EL))
+	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
+	if (wp->xoff == 0 && screen_size_x(s) >= tty->sx &&
+	    tty_term_has(tty->term, TTYC_EL))
 		tty_putcode(tty, TTYC_EL);
 	else {
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
-		for (i = s->cx; i < screen_size_x(s); i++)
+		for (i = s->old_cx; i < screen_size_x(s); i++)
 			tty_putc(tty, ' ');
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 	}
 }
 
 void
 tty_cmd_clearstartofline(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+    struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i;
+	struct screen	*s = wp->screen;
+	u_int		 i;
 
 	tty_reset(tty);
- 	tty_cursor(tty, s->cx, s->cy, oy);
 
-	if (tty_term_has(tty->term, TTYC_EL1))
+	if (wp->xoff == 0 && tty_term_has(tty->term, TTYC_EL1)) {
+		tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 		tty_putcode(tty, TTYC_EL1);
-	else {
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, 0);
-		for (i = 0; i < s->cx + 1; i++)
+	} else {
+		tty_cursor(tty, 0, s->old_cy, wp->xoff, wp->yoff);
+		for (i = 0; i < s->old_cx + 1; i++)
 			tty_putc(tty, ' ');
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 	}
 }
 
 void
-tty_cmd_reverseindex(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+tty_cmd_reverseindex(struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	tty_reset(tty);
- 	tty_region(tty, s, oy);
-	tty_cursor(tty, s->cx, s->cy, oy);
+	struct screen	*s = wp->screen;
 
-	tty_putcode(tty, TTYC_RI);
+ 	if (wp->xoff != 0 || screen_size_x(s) < tty->sx ||
+	    !tty_term_has(tty->term, TTYC_CSR)) {
+		tty_redraw_region(tty, wp);
+		return;
+	}
+
+	tty_reset(tty);
+
+ 	tty_region(tty, s->old_rupper, s->old_rlower, wp->yoff);
+
+	if (s->old_cy == s->old_rupper) {
+		tty_cursor(tty, s->old_cx, s->old_rupper, wp->xoff, wp->yoff);
+		tty_putcode(tty, TTYC_RI);
+	}
 }
 
 void
-tty_cmd_linefeed(struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+tty_cmd_linefeed(struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	tty_reset(tty);
- 	tty_region(tty, s, oy);
-	tty_cursor(tty, s->cx, s->cy, oy);
+	struct screen	*s = wp->screen;
 
+ 	if (wp->xoff != 0 || screen_size_x(s) < tty->sx ||
+	    !tty_term_has(tty->term, TTYC_CSR)) {
+		tty_redraw_region(tty, wp);
+		return;
+	}
+
+	tty_reset(tty);
+
+ 	tty_region(tty, s->old_rupper, s->old_rlower, wp->yoff);
+
+	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
 	tty_putc(tty, '\n');
+
 	tty->cy++;
 }
 
-void
-tty_cmd_bell(struct tty *tty,
-    unused struct screen *s, unused u_int oy, unused va_list ap)
-{
-	tty_putcode(tty, TTYC_BEL);
-}
-
 void
 tty_cmd_clearendofscreen(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+    struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i, j;
+	struct screen	*s = wp->screen;
+	u_int		 i, j, oy;
+
+	oy = wp->yoff;
 
 	tty_reset(tty);
-	tty_cursor(tty, s->cx, s->cy, oy);
 
-	if (tty_term_has(tty->term, TTYC_EL)) {
-		for (i = oy + s->cy; i < oy + screen_size_y(s); i++) {
-			tty_putcode(tty, TTYC_EL);
-			if (i != screen_size_y(s) - 1)
+	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
+	tty_region(tty, 0, screen_size_y(s) - 1, wp->yoff);
+	if (wp->xoff == 0 && screen_size_x(s) >= tty->sx &&
+	    tty_term_has(tty->term, TTYC_EL)) {	
+		tty_putcode(tty, TTYC_EL);
+		if (s->old_cy != screen_size_y(s) - 1) {
+			tty_cursor(tty, 0, s->old_cy + 1, wp->xoff, wp->yoff);
+			for (i = s->old_cy + 1; i < screen_size_y(s); i++) {
+				tty_putcode(tty, TTYC_EL);
+				if (i == screen_size_y(s) - 1)
+					continue;
 				tty_emulate_repeat(tty, TTYC_CUD, TTYC_CUD1, 1);
+				tty->cy++;
+			}
 		}
 	} else {
-		for (i = s->cx; i < screen_size_y(s); i++)
+		for (i = s->old_cx; i < screen_size_x(s); i++)
 			tty_putc(tty, ' ');
-		for (j = oy + s->cy; j < oy + screen_size_y(s); j++) {
+		for (j = s->old_cy; j < screen_size_y(s); j++) {
+			tty_cursor(tty, 0, j, wp->xoff, wp->yoff);
 			for (i = 0; i < screen_size_x(s); i++)
 				tty_putc(tty, ' ');
 		}
 	}
-	tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 }
 
 void
 tty_cmd_clearstartofscreen(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+    struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i, j;
+	struct screen	*s = wp->screen;
+	u_int		 i, j;
 
 	tty_reset(tty);
-	tty_cursor(tty, s->cx, s->cy, oy);
 
-	tty_putcode2(tty, TTYC_CUP, oy, 0);
-	if (tty_term_has(tty->term, TTYC_EL)) {
-		for (i = 0; i < oy + s->cy; i++) {
+	tty_cursor(tty, 0, 0, wp->xoff, wp->yoff);
+	tty_region(tty, 0, screen_size_y(s) - 1, wp->yoff);
+	if (wp->xoff == 0 && screen_size_x(s) >= tty->sx &&
+	    tty_term_has(tty->term, TTYC_EL)) {
+		for (i = 0; i < s->old_cy; i++) {
 			tty_putcode(tty, TTYC_EL);
 			tty_emulate_repeat(tty, TTYC_CUD, TTYC_CUD1, 1);
+			tty->cy++;
 		}
-		tty_putcode2(tty, TTYC_CUP, oy + s->cy, 0);
 	} else {
-		for (j = 0; j < oy + s->cy; j++) {
+		for (j = 0; j < s->old_cy; j++) {
+			tty_cursor(tty, 0, j, wp->xoff, wp->yoff);
 			for (i = 0; i < screen_size_x(s); i++)
 				tty_putc(tty, ' ');
 		}
 	}
-	for (i = 0; i < s->cx; i++)
+	for (i = 0; i < s->old_cx; i++)
 		tty_putc(tty, ' ');
-	tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 }
 
 void
 tty_cmd_clearscreen(
-    struct tty *tty, struct screen *s, u_int oy, unused va_list ap)
+    struct tty *tty, struct window_pane *wp, unused va_list ap)
 {
-	u_int	i, j;
+	struct screen	*s = wp->screen;
+	u_int		 i, j;
 
 	tty_reset(tty);
-	tty_cursor(tty, s->cx, s->cy, oy);
 
-	tty_putcode2(tty, TTYC_CUP, oy, 0);
-	if (tty_term_has(tty->term, TTYC_EL)) {
+	tty_cursor(tty, 0, 0, wp->xoff, wp->yoff);
+	tty_region(tty, 0, screen_size_y(s) - 1, wp->yoff);
+	if (wp->xoff == 0 && screen_size_x(s) >= tty->sx &&
+	    tty_term_has(tty->term, TTYC_EL)) {
 		for (i = 0; i < screen_size_y(s); i++) {
 			tty_putcode(tty, TTYC_EL);
-			if (i != screen_size_y(s) - 1)
+			if (i != screen_size_y(s) - 1) {
 				tty_emulate_repeat(tty, TTYC_CUD, TTYC_CUD1, 1);
+				tty->cy++;
+			}
 		}
 	} else {
 		for (j = 0; j < screen_size_y(s); j++) {
+			tty_cursor(tty, 0, j, wp->xoff, wp->yoff);
 			for (i = 0; i < screen_size_x(s); i++)
 				tty_putc(tty, ' ');
 		}
 	}
-	tty_putcode2(tty, TTYC_CUP, oy + s->cy, s->cx);
 }
 
 void
-tty_cmd_cell(struct tty *tty, struct screen *s, u_int oy, va_list ap)
+tty_cmd_cell(struct tty *tty, struct window_pane *wp, va_list ap)
 {
-	struct grid_cell       *gc;
-	u_int			i, width;
-	u_char			out[4];
-
-	tty_cursor(tty, s->cx, s->cy, oy);
+	struct screen		*s = wp->screen;
+	struct grid_cell	*gc;
+	struct grid_utf8	*gu;
 
 	gc = va_arg(ap, struct grid_cell *);
+	gu = va_arg(ap, struct grid_utf8 *);
+
+	tty_cursor(tty, s->old_cx, s->old_cy, wp->xoff, wp->yoff);
+
+	tty_cell(tty, gc, gu);
+}
+
+void
+tty_cmd_raw(struct tty *tty, unused struct window_pane *wp, va_list ap)
+{
+	u_char	*buf;
+	size_t	 i, len;
+
+	buf = va_arg(ap, u_char *);
+	len = va_arg(ap, size_t);
+
+	for (i = 0; i < len; i++)
+		tty_putc(tty, buf[i]);
+}
+
+void
+tty_cell(
+    struct tty *tty, const struct grid_cell *gc, const struct grid_utf8 *gu)
+{
+	u_int	i;
+
+	/* Skip last character if terminal is stupid. */
+	if (tty->term->flags & TERM_EARLYWRAP && 
+	    tty->cy == tty->sy - 1 && tty->cx == tty->sx - 1)
+		return;
 
 	/* If this is a padding character, do nothing. */
 	if (gc->flags & GRID_FLAG_PADDING)
@@ -647,7 +802,7 @@ tty_cmd_cell(struct tty *tty, struct screen *s, u_int oy, va_list ap)
 
 	/* If not UTF-8, write directly. */
 	if (!(gc->flags & GRID_FLAG_UTF8)) {
-		if (gc->data > 0xff || gc->data < 0x20 || gc->data == 0x7f)
+		if (gc->data < 0x20 || gc->data == 0x7f)
 			return;
 		tty_putc(tty, gc->data);
 		return;
@@ -655,56 +810,55 @@ tty_cmd_cell(struct tty *tty, struct screen *s, u_int oy, va_list ap)
 
 	/* If the terminal doesn't support UTF-8, write underscores. */
 	if (!(tty->flags & TTY_UTF8)) {
-		width = utf8_width(gc->data);
-		while (width-- > 0)
+		for (i = 0; i < gu->width; i++)
 			tty_putc(tty, '_');
 		return;
 	}
 
-	/* Otherwise, unpack UTF-8 and write it. */
-	utf8_split(gc->data, out);
-	for (i = 0; i < 4; i++) {
-		if (out[i] == 0xff)
+	/* Otherwise, write UTF-8. */
+	for (i = 0; i < UTF8_SIZE; i++) {
+		if (gu->data[i] == 0xff)
 			break;
-		tty_putc(tty, out[i]);
+		tty_putc(tty, gu->data[i]);
 	}
 }
 
 void
 tty_reset(struct tty *tty)
 {
-	struct grid_cell	*tc = &tty->cell;
+	struct grid_cell	*gc = &tty->cell;
 
-	tc->data = grid_default_cell.data;
-	if (memcmp(tc, &grid_default_cell, sizeof *tc) == 0)
+	if (memcmp(gc, &grid_default_cell, sizeof *gc) == 0)
 		return;
 
-	if (tty_term_has(tty->term, TTYC_RMACS) && tc->attr & GRID_ATTR_CHARSET)
+	if (tty_term_has(tty->term, TTYC_RMACS) && gc->attr & GRID_ATTR_CHARSET)
 		tty_putcode(tty, TTYC_RMACS);
 	tty_putcode(tty, TTYC_SGR0);
-	memcpy(tc, &grid_default_cell, sizeof *tc);
+	memcpy(gc, &grid_default_cell, sizeof *gc);
 }
 
 void
-tty_region(struct tty *tty, struct screen *s, u_int oy)
+tty_region(struct tty *tty, u_int rupper, u_int rlower, u_int oy)
 {
-	if (tty->rlower != oy + s->rlower || tty->rupper != oy + s->rupper) {
-		tty->rlower = oy + s->rlower;
-		tty->rupper = oy + s->rupper;
+	if (!tty_term_has(tty->term, TTYC_CSR))
+		return;
+	if (tty->rlower != oy + rlower || tty->rupper != oy + rupper) {
+		tty->rlower = oy + rlower;
+		tty->rupper = oy + rupper;
 		tty->cx = 0;
-		tty->cy = 0;
+	 	tty->cy = 0;
 		tty_putcode2(tty, TTYC_CSR, tty->rupper, tty->rlower);
 	}
 }
 
 void
-tty_cursor(struct tty *tty, u_int cx, u_int cy, u_int oy)
+tty_cursor(struct tty *tty, u_int cx, u_int cy, u_int ox, u_int oy)
 {
-	if (cx == 0 && tty->cx != 0 && tty->cy == oy + cy) {
+	if (ox + cx == 0 && tty->cx != 0 && tty->cy == oy + cy) {
 		tty->cx = 0;
 		tty_putc(tty, '\r');
-	} else if (tty->cx != cx || tty->cy != oy + cy) {
-		tty->cx = cx;
+	} else if (tty->cx != ox + cx || tty->cy != oy + cy) {
+		tty->cx = ox + cx;
 		tty->cy = oy + cy;
 		tty_putcode2(tty, TTYC_CUP, tty->cy, tty->cx);
 	}
@@ -764,20 +918,47 @@ tty_attributes(struct tty *tty, const struct grid_cell *gc)
 	}
 }
 
+int
+tty_try_256(struct tty *tty, u_char colour, const char *type)
+{
+	char	s[32];
+
+	if (!(tty->term->flags & TERM_256COLOURS) && 
+	    !(tty->term_flags & TERM_256COLOURS))
+		return (-1);
+	
+	xsnprintf(s, sizeof s, "\033[%s;5;%hhum", type, colour);
+	tty_puts(tty, s);
+	return (0);
+}
+
+int
+tty_try_88(struct tty *tty, u_char colour, const char *type)
+{
+	char	s[32];
+	
+	if (!(tty->term->flags & TERM_88COLOURS) &&
+	    !(tty->term_flags & TERM_88COLOURS))
+		return (-1);
+	colour = colour_256to88(colour);
+
+	xsnprintf(s, sizeof s, "\033[%s;5;%hhum", type, colour);
+	tty_puts(tty, s);
+	return (0);
+}
+
 void
 tty_attributes_fg(struct tty *tty, const struct grid_cell *gc)
 {
-	char	s[32];
-	u_char	fg = gc->fg;
+	u_char	fg;
 
+	fg = gc->fg;
 	if (gc->flags & GRID_FLAG_FG256) {
-		if ((tty->term->flags & TERM_256COLOURS) ||
-		    (tty->term_flags & TERM_256COLOURS)) {
-			xsnprintf(s, sizeof s, "\033[38;5;%hhum", fg);
-			tty_puts(tty, s);
+		if (tty_try_256(tty, fg, "38") == 0)
 			return;
-		}
-		fg = colour_translate256(fg);
+		if (tty_try_88(tty, fg, "38") == 0)
+			return;
+		fg = colour_256to16(fg);
 		if (fg & 8) {
 			fg &= 7;
 			tty_putcode(tty, TTYC_BOLD);
@@ -785,7 +966,7 @@ tty_attributes_fg(struct tty *tty, const struct grid_cell *gc)
 		} else if (tty->cell.attr & GRID_ATTR_BRIGHT)
 			tty_reset(tty);
 	}
-
+	
 	if (fg == 8 &&
 	    !(tty->term->flags & TERM_HASDEFAULTS) &&
 	    !(tty->term_flags & TERM_HASDEFAULTS))
@@ -799,29 +980,17 @@ tty_attributes_fg(struct tty *tty, const struct grid_cell *gc)
 void
 tty_attributes_bg(struct tty *tty, const struct grid_cell *gc)
 {
-	char	s[32];
-	u_char	bg = gc->bg;
+	u_char	bg;
 
+	bg = gc->bg;
 	if (gc->flags & GRID_FLAG_BG256) {
-		if ((tty->term->flags & TERM_256COLOURS) ||
-		    (tty->term_flags & TERM_256COLOURS)) {
-			xsnprintf(s, sizeof s, "\033[48;5;%hhum", bg);
-			tty_puts(tty, s);
+		if (tty_try_256(tty, bg, "48") == 0)
 			return;
-		}
-		bg = colour_translate256(bg);
-		if (bg & 8) {
-			/*
-			 * Bold background; can't do this on standard
-			 * terminals...
-			 */
-#if 0
-			xsnprintf(s, sizeof s, "\033[%hhum", 92 + bg);
-			tty_puts(tty, s);
+		if (tty_try_88(tty, bg, "48") == 0)
 			return;
-#endif
+		bg = colour_256to16(bg);
+		if (bg & 8)
 			bg &= 7;
-		}
 	}
 
 	if (bg == 8 &&
