@@ -1,4 +1,4 @@
-/* $Id: window-copy.c,v 1.49 2009/01/28 22:00:22 nicm Exp $ */
+/* $Id: window-copy.c,v 1.59 2009/03/31 15:10:43 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -73,6 +73,8 @@ const struct window_mode window_copy_mode = {
 struct window_copy_mode_data {
 	struct screen	screen;
 
+	struct mode_key_data	mdata;
+
 	u_int	ox;
 	u_int	oy;
 
@@ -101,6 +103,9 @@ window_copy_init(struct window_pane *wp)
 	screen_init(s, screen_size_x(&wp->base), screen_size_y(&wp->base), 0);
 	s->mode |= MODE_MOUSE;
 
+	mode_key_init(&data->mdata,
+	    options_get_number(&wp->window->options, "mode-keys"), 0);
+
 	s->cx = data->cx;
 	s->cy = data->cy;
 
@@ -118,6 +123,8 @@ window_copy_free(struct window_pane *wp)
 {
 	struct window_copy_mode_data	*data = wp->modedata;
 
+ 	mode_key_free(&data->mdata);
+
 	screen_free(&data->screen);
 	xfree(data);
 }
@@ -155,29 +162,27 @@ window_copy_key(struct window_pane *wp, struct client *c, int key)
 {
 	struct window_copy_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
-	int				 table;
 
-	table = options_get_number(&wp->window->options, "mode-keys");
-	switch (mode_key_lookup(table, key)) {
-	case MODEKEY_QUIT:
+	switch (mode_key_lookup(&data->mdata, key)) {
+	case MODEKEYCMD_QUIT:
 		window_pane_reset_mode(wp);
 		break;
-	case MODEKEY_LEFT:
+	case MODEKEYCMD_LEFT:
 		window_copy_cursor_left(wp);
 		return;
-	case MODEKEY_RIGHT:
+	case MODEKEYCMD_RIGHT:
 		window_copy_cursor_right(wp);
  		return;
-	case MODEKEY_UP:
+	case MODEKEYCMD_UP:
 		window_copy_cursor_up(wp);
 		return;
-	case MODEKEY_DOWN:
+	case MODEKEYCMD_DOWN:
 		window_copy_cursor_down(wp);
 		return;
-	case MODEKEY_PPAGE:
+	case MODEKEYCMD_PREVIOUSPAGE:
 		window_copy_pageup(wp);
 		break;
-	case MODEKEY_NPAGE:
+	case MODEKEYCMD_NEXTPAGE:
 		if (data->oy < screen_size_y(s))
 			data->oy = 0;
 		else
@@ -185,30 +190,29 @@ window_copy_key(struct window_pane *wp, struct client *c, int key)
 		window_copy_update_selection(wp);
 		window_copy_redraw_screen(wp);
 		break;
-	case MODEKEY_STARTSEL:
+	case MODEKEYCMD_STARTSELECTION:
  		window_copy_start_selection(wp);
 		break;
-	case MODEKEY_CLEARSEL:
+	case MODEKEYCMD_CLEARSELECTION:
 		screen_clear_selection(&data->screen);
 		window_copy_redraw_screen(wp);
 		break;
-	case MODEKEY_COPYSEL:
-	case MODEKEY_ENTER:
+	case MODEKEYCMD_COPYSELECTION:
 		if (c != NULL && c->session != NULL) {
 			window_copy_copy_selection(wp, c);
 			window_pane_reset_mode(wp);
 		}
 		break;
-	case MODEKEY_BOL:
+	case MODEKEYCMD_STARTOFLINE:
 		window_copy_cursor_start_of_line(wp);
 		break;
-	case MODEKEY_EOL:
+	case MODEKEYCMD_ENDOFLINE:
 		window_copy_cursor_end_of_line(wp);
 		break;
-	case MODEKEY_NWORD:
+	case MODEKEYCMD_NEXTWORD:
 		window_copy_cursor_next_word(wp);
 		break;
-	case MODEKEY_PWORD:
+	case MODEKEYCMD_PREVIOUSWORD:
 		window_copy_cursor_previous_word(wp);
 		break;
 	default:
@@ -256,7 +260,6 @@ window_copy_write_line(struct window_pane *wp, struct screen_write_ctx *ctx, u_i
 		gc.attr |= options_get_number(&wp->window->options, "mode-attr");
 		screen_write_cursormove(ctx, screen_size_x(s) - size, 0);
 		screen_write_puts(ctx, &gc, "%s", hdr);
-		screen_write_puts(ctx, &gc, "%s", hdr);
 	} else
 		size = 0;
 
@@ -284,7 +287,7 @@ window_copy_write_column(
 
 	screen_write_cursormove(ctx, px, 0);
 	screen_write_copy(ctx, &wp->base,
-	    data->ox, screen_hsize(&wp->base) - data->oy, 1, screen_size_y(s));
+	    data->ox + px, screen_hsize(&wp->base) - data->oy, 1, screen_size_y(s));
 }
 
 void
@@ -294,7 +297,7 @@ window_copy_write_columns(
 	u_int	xx;
 
 	for (xx = px; xx < px + nx; xx++)
-		window_copy_write_column(wp, ctx, px);
+		window_copy_write_column(wp, ctx, xx);
 }
 
 void
@@ -458,12 +461,12 @@ window_copy_copy_selection(struct window_pane *wp, struct client *c)
 }
 
 void
-window_copy_copy_line(
-    struct window_pane *wp, char **buf, size_t *off, u_int sy, u_int sx, u_int ex)
+window_copy_copy_line(struct window_pane *wp,
+    char **buf, size_t *off, u_int sy, u_int sx, u_int ex)
 {
-	const struct grid_cell *gc;
-	u_int			i, j, xx;
-	u_char			data[4];
+ 	const struct grid_cell	*gc;
+ 	const struct grid_utf8	*gu;
+	u_int			 i, j, xx;
 
 	if (sx > ex)
 		return;
@@ -477,13 +480,19 @@ window_copy_copy_line(
 	if (sx < ex) {
 		for (i = sx; i < ex; i++) {
 			gc = grid_peek_cell(wp->base.grid, i, sy);
-			utf8_split(gc->data, data);
-
-			*buf = xrealloc(*buf, 1, (*off) + 4);
-			for (j = 0; j < sizeof data; j++) {
-				if (data[j] == 0xff)
-					break;
-				(*buf)[(*off)++] = data[j];
+			if (gc->flags & GRID_FLAG_PADDING)
+				continue;
+			if (!(gc->flags & GRID_FLAG_UTF8)) {
+				*buf = xrealloc(*buf, 1, (*off) + 1);
+				(*buf)[(*off)++] = gc->data;
+			} else {
+				gu = grid_peek_utf8(wp->base.grid, i, sy);
+				*buf = xrealloc(*buf, 1, (*off) + UTF8_SIZE);
+				for (j = 0; j < UTF8_SIZE; j++) {
+					if (gu->data[j] == 0xff)
+						break;
+					(*buf)[(*off)++] = gu->data[j];
+				}
 			}
 		}
 	}
@@ -497,12 +506,12 @@ int
 window_copy_is_space(struct window_pane *wp, u_int px, u_int py)
 {
 	const struct grid_cell	*gc;
-	const char 		*spaces = " -_@";
+	const char     		*spaces = " -_@";
 
 	gc = grid_peek_cell(wp->base.grid, px, py);
-	if (gc->flags & GRID_FLAG_PADDING)
+	if (gc->flags & (GRID_FLAG_PADDING|GRID_FLAG_UTF8))
 		return (0);
-	if (gc->data == 0x00 || gc->data > 0xff)
+	if (gc->data == 0x00 || gc->data == 0x7f)
 		return (0);
 	return (strchr(spaces, gc->data) != NULL);
 }
@@ -516,7 +525,9 @@ window_copy_find_length(struct window_pane *wp, u_int py)
 	px = wp->base.grid->size[py];
 	while (px > 0) {
 		gc = grid_peek_cell(wp->base.grid, px - 1, py);
-		if (gc->data != 0x20)
+		if (gc->flags & GRID_FLAG_UTF8)
+			break;
+		if (gc->data != ' ')
 			break;
 		px--;
 	}
