@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.64 2009/01/28 19:52:21 nicm Exp $ */
+/* $Id: window.c,v 1.72 2009/04/01 18:46:03 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -201,8 +201,7 @@ window_index(struct window *s, u_int *i)
 }
 
 struct window *
-window_create(const char *name, const char *cmd, const char *cwd,
-    const char **envp, u_int sx, u_int sy, u_int hlimit, char **cause)
+window_create1(u_int sx, u_int sy)
 {
 	struct window	*w;
 	u_int		 i;
@@ -213,7 +212,7 @@ window_create(const char *name, const char *cmd, const char *cwd,
 
 	TAILQ_INIT(&w->panes);
 	w->active = NULL;
-	w->pgrp = -1;
+	w->layout = 0;
 
 	w->sx = sx;
 	w->sy = sy;
@@ -230,6 +229,16 @@ window_create(const char *name, const char *cmd, const char *cwd,
 		ARRAY_ADD(&windows, w);
 	w->references = 0;
 
+	return (w);
+}
+
+struct window *
+window_create(const char *name, const char *cmd, const char *cwd,
+    const char **envp, u_int sx, u_int sy, u_int hlimit, char **cause)
+{
+	struct window	*w;
+
+	w = window_create1(sx, sy);
 	if (window_add_pane(w, -1, cmd, cwd, envp, hlimit, cause) == NULL) {
 		window_destroy(w);
 		return (NULL);
@@ -367,6 +376,7 @@ window_update_panes(struct window *w)
 	TAILQ_FOREACH(wp, &w->panes, entry) {
 		if (wp->flags & PANE_HIDDEN)
 			continue;
+		wp->xoff = 0;
 		wp->yoff = yoff;
 		yoff += wp->sy + 1;
 	}
@@ -391,8 +401,10 @@ window_add_pane(struct window *w, int wanty, const char *cmd,
 		wanty = w->sy;
 	else {
 		sizey = w->active->sy - 1; /* for separator */
-		if (sizey < PANE_MINIMUM * 2)
+		if (sizey < PANE_MINIMUM * 2) {
+			*cause = xstrdup("pane too small");
 			return (NULL);
+		}
 
   		if (wanty == -1)
 			wanty = sizey / 2;
@@ -502,11 +514,12 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 
 	wp->mode = NULL;
 
+	wp->xoff = 0;
+ 	wp->yoff = 0;
+
 	wp->sx = sx;
 	wp->sy = sy;
 
- 	wp->yoff = 0;
-
 	screen_init(&wp->base, sx, sy, hlimit);
 	wp->screen = &wp->base;
 
@@ -562,14 +575,13 @@ window_pane_spawn(struct window_pane *wp,
 	ws.ws_col = screen_size_x(&wp->base);
 	ws.ws_row = screen_size_y(&wp->base);
 
-	wp->window->pgrp = -1;
 	if (gettimeofday(&wp->window->name_timer, NULL) != 0)
 		fatal("gettimeofday");
 	tv.tv_sec = 0;
 	tv.tv_usec = NAME_INTERVAL * 1000L;
 	timeradd(&wp->window->name_timer, &tv, &wp->window->name_timer);
 
- 	switch (forkpty(&wp->fd, wp->tty, NULL, &ws)) {
+ 	switch (wp->pid = forkpty(&wp->fd, wp->tty, NULL, &ws)) {
 	case -1:
 		wp->fd = -1;
 		xasprintf(cause, "%s: %s", cmd, strerror(errno));
@@ -672,8 +684,11 @@ window_pane_mouse(
 {
 	/* XXX convert from 1-based? */
 
+	if (x < wp->xoff || x >= wp->xoff + wp->sx)
+		return;
 	if (y < wp->yoff || y >= wp->yoff + wp->sy)
 		return;
+	x -= wp->xoff;
 	y -= wp->yoff;
 
 	if (wp->mode != NULL) {
